var vmware = (typeof vmware == "undefined" || !vmware) ? {} : vmware;

/**
 * Normalize the constants defined on a provided plugin instance.
 *
 * Post-normalization, constants can be accessed like:
 *     vmware.Mode.MKS
 *     vmware.ConnectionState.CS_CONNECTED
 *
 * @param {object} vmrc The VMRC plugin object.
 * @return {object} The same VMRC plugin object with normalized constants.
 */
vmware.normalizeConstants = function (vmrc) {
    var isMsie = $.browser.msie;

    /**
     * Remove the "VMRC_" prefix; the "VMRC_" scoping is implied.
     *
     * @param {string} The string from which the prefix should be removed.
     * @return {string} The string with the prefix removed (if present).
     */
    var shorten = function (input) {
        return input.replace(/VMRC_/g, "");
    };

    /**
     * Convert a MSIE-style property into a standard JS object
     *
     * @param {object} property The property to convert.
     * @return {object} A normalized object which can be used as an assoicative array.
     */
    var getNormalizedProperty = function (property) {
        if (isMsie) {
            var keys = {};
            var vbkeys = new VBArray(property.Keys());
            for (k = 0; k <= vbkeys.ubound(1); k++) {
                key = vbkeys.getItem(k);
                keys[key] = property(key);
            }
            return keys;
        } else {
            return property;
        }
    };

    /**
     * Create a "short key"-to-value map for all keys for a given property
     *
     * @param {object} normalizedProperty An associative array-like object for which the keys should be shortened.
     * @return {object} A new associative array-like object with shortened keys.
     */
    var shortenProperty = function (normalizedProperty) {
        var values = {};
        for (var key in normalizedProperty) {
            values[shorten(key)] = normalizedProperty[key];
        }

        return values;
    };

    /**
     * Create a "short property"-to-("short key"-to-value map) map
     *
     * @param {object} pluginInstance The VMRC plugin object on which to operate.
     * @return {object} An associative array-like object of shortened property names and key names.
     */
    var getConstants = function (pluginInstance) {
        var constants = {};

        constants["ConnectionState"] = {CS_DISCONNECTED: 1, CS_CONNECTED: 2};
        constants["DeviceBacking"] = {DB_FILE: 1, DB_PHYSICAL: 2};
        constants["DeviceState"] = {DS_DISCONNECTED: 1, DS_CONNECTED: 2};
        constants["DeviceType"] = {DEVICE_FLOPPY: 1, DEVICE_CDROM: 2, DEVICE_USB: 4, DEVICE_ALL: 7};
        constants["GrabState"] = {GS_GRABBED: 1, GS_UNGRABBED_HARD: 2, GS_UNGRABBED_SOFT: 3};
        constants["MessageMode"] = {DIALOG_MESSAGES: 1, EVENT_MESSAGES: 2, NO_MESSAGES: 3};
        constants["MessageType"] = {MESSAGE_INFO: 1, MESSAGE_WARNING: 2, MESSAGE_ERROR: 3, MESSAGE_HINT: 4};
        constants["Mode"] = {MKS: 2, DEVICES: 4};
        constants["USBDeviceFamily"] = {USB_FAMILY_AUDIO: 8, USB_FAMILY_BLUETOOTH: 65536, USB_FAMILY_COMM: 16, USB_FAMILY_HID: 32,
                USB_FAMILY_HID_BOOTABLE: 64, USB_FAMILY_IMAGING: 256, USB_FAMILY_OTHER: 4, USB_FAMILY_PDA: 262144, USB_FAMILY_PHYSICAL: 128,
                USB_FAMILY_PRINTER: 512, USB_FAMILY_SECURITY: 8192, USB_FAMILY_SMART_CARD: 4096, USB_FAMILY_STORAGE: 1024,
                USB_FAMILY_UNKNOWN: 2, USB_FAMILY_VENDOR: 1, USB_FAMILY_VIDEO: 16384, USB_FAMILY_WIRELESS: 32768, USB_FAMILY_WUSB: 131072};
        constants["USBDeviceSpeed"] = {USB_SPEED_FULL: 2, USB_SPEED_HIGH: 4, USB_SPEED_LOW: 1, USB_SPEED_SUPER: 8, USB_SPEED_UNKNOWN: -2147483648};
        return constants;
    };

    /**
     * Take a map, like the one returned from getConstants, and add the values to the given object.
     *
     * This is similar to jQuery.extend, but compatible with the plugin.
     *
     * @param {object} object The object to extend.
     * @param {object} constants The constants to add.
     */
    var applyConstantsToObject = function (object, constants) {
        for (var propertyName in constants) {
             object[propertyName] = constants[propertyName];
        }
        return object;
    };

    var constants = getConstants(vmrc);

    return applyConstantsToObject(vmrc, constants);
};
