<#
.SYNOPSIS
mswdiags.ps1 is designed to take a VMware vCenter deployment with Metaswitch VM
products running on it and report back specific configuration settings of VMs,
hosts and clusters in the vCenter.

.DESCRIPTION
The output of the script is:
  - The HTML document "analysis_report.html" containing analysis of settings
    within the vSphere and whether they are compliant with the VDDG. This is
    created by the function createAnalysisReport.
  - A .txt file for each cluster and host with raw information about important
    settings. This is created by the dump method.
  - A .vmx file for all Metaswitch VMs containing detailed configuration
    settings. This is created by the dump method.
  - Two .csv files containing all VMs and all hosts within the cluster
    respectively. These contains basic setting information and are produced by
    the functions VMdetails and Hostdetails.
  - if the -collectlogs parameter is passed when the script is run then large
    .tgx log files will be collected.
  - if the vnow parameter is passed in the script checks the deployment is
    correctly configured for vNOW
  All files are placed within destdir.

Copyright (C) Metaswitch Networks

.PARAMETER vCenter
This parameter is the IP address or domain name of the vCenter the script
is being run on.

.PARAMETER destdir
This parameter is the name of the destination directory you want all log
files to be placed. The name must be unique.

.PARAMETER clustername
This parameter can be used to specify the name of specific clusters in the
vCenter. Only diagnostics about these clusters and the hosts and VMs within them
will be collected.

.PARAMETER vmName
This parameter can be used to specify individual VMs, only diagnostics
about these VMs and the clusters they reside in will be collected.

.PARAMETER collectlogs
If -collectlogs is passed as an argument then large detailed support logs are
collected about the vCenter

.PARAMETER vnow
If -vnow is passed in the script checks the deployment is correctly configured
for vNOW
#>

param([String]$vCenter,
      [String]$destDir,
      [String]$clusterName="",
      [String]$vmName="",
      [switch]$collectLogs = $FALSE,
      [switch]$vnow = $FALSE)

# mswdiags.ps1 goes through each cluster on a vCenter and creates an object for
# the cluster and each host and VM on it. The script has separate classes of
# objects for clusters, hosts and VMs. For cluster and VM objects there are
# specific child classes that are used depending on the type of cluster/VM.
# The following methods are used on each object:
#
# getInfo - A cluster, host or VM is passed as a parameter to the method. It is
#   then used to populate the new object the method is called on with relevant
#   diagnostic information.
#
# analysis - The information within the object is compared to the recommended
#   configuration settings detailed in the VMware Deployment Design Guide
#   (VDDG). If a setting is non compliant an object property is updated to
#   reflect that.
#
# gatherAnalysisHTML - This method builds up and returns a string of HTML text
#   containing all non-compliant settings and advice on changing the settings.
#   This text is used to create a HTML analysis document.
#
# gatherAnalysisTable - This method takes information on config settings from
#   the object and stores it in an array. Each element of the array contains
#   another array detailing:
#     - The name of the setting
#     - What the setting is.
#     - Whether that setting is compliant or not.
#
# dump - This method returns a text file containing raw unanalysed information
#   about a cluster or host, or a .vmx file containing detailed information for
#   Metaswitch VMs.

# VApp vendor and product name strings
$vmMswVendor = "Metaswitch Networks"
$vmNameCommonString = "Metaswitch"
$vmName_ABU_RPAS = "Metaswitch Virtual Appliance: Representative Application Server"
$vmName_ABU_ESAP = "Metaswitch Virtual Appliance: ESA Proxy Server"
$vmName_ABU_AMS  = "Metaswitch Virtual Appliance: Accession Messaging Service"
$vmName_ABU_OBS  = "Metaswitch Virtual Appliance: Object Backup Store"
$vmName_ABU_SAS  = "Metaswitch Virtual Appliance: Service Assurance Server"
$vmName_ABU_MVS  = "Metaswitch Virtual Appliance: MetaView Server"
$vmName_ABU_MVOS = "Metaswitch Virtual Appliance: MetaView OS"
$vmName_ABU_MVD  = "Metaswitch Virtual Appliance: MetaView Director"
$vmName_ABU_CFS  = "Metaswitch Virtual Appliance: Call Feature Server"
$vmName_ABU_EAS  = "Metaswitch Virtual Appliance: Enhanced Application Server"
$vmName_ABU_MRS  = "Metaswitch Virtual Appliance: Media Resource Server"
$vmName_SBU_Perimeta = "Metaswitch Virtual Appliance: Perimeta"
$vmName_SBU_DCM  = "Metaswitch Distributed Capacity Manager"

################################################################################
# Class ParentClusterClass
#
# ParentClusterClass should be treated as an abstract parent class and must not
# be instantiated. The child classes ClusterClass and GhostClusterClass should
# be used to both create cluster objects that hold information about the cluster
# (if there is one), and hold objects about all VMs and Hosts within it.
# This parent class contains methods that are used by the child classes to:
#   Get all relevant info about the host and VM objects (getInfo).
#   Analyse that info (analysis).
#   Collate that analysis so it can be used to generate an HTML analysis
#     document (gatherAnalysisHTML and gatherAnalysisTable).
#   Output that information in a set of files (dump).
#
# Each cluster object contains two arrays which contain the VM objects and host
# objects within that cluster. When the getInfo, analysis, gatherAnalysisHTML or
# dump methods are run on a cluster the method runs the analogous methods on each
# VM and host within that cluster.
################################################################################
$ParentClusterClass = new-object psobject -Property @{
  cluster = $null
  clusterIssues = $null

  # The follwing are the arrays of the HostObjects and VMObjects.
  HostArray = @()
  VMArray = @()

  # nonPerimetaVMsPresent notes whether the cluster has any non-Perimeta VMs.
  nonPerimetaVMsPresent = $FALSE
}

# analysisParent runs methods to analyse particular properties of the VM and
# host objects in the cluster object to see if they are compliant with the
# VMware Deployment Design guide. analysisParent is called by the analysis
# method in the child classes.
$ParentClusterClass | Add-Member -MemberType ScriptMethod -Name "analysisParent" -Value {

  foreach ($hostObject in $this.HostArray)
  {
    $this.clusterIssues += $hostObject.analysis($this.VMarray)
  }

  foreach ($VMObject in $this.VMArray)
  {
    $this.clusterIssues += $VMObject.analysis()

    if (!$VMObject.isPerimeta)
    {
      $this.nonPerimetaVMsPresent = $TRUE
    }
  }
}

# gatherAnalysisHTMLParent runs method to return an HTML string of all non
# compliant settings that have been detected on host and VM objects.
$ParentClusterClass | Add-Member -MemberType ScriptMethod -Name "gatherAnalysisHTMLParent" -Value {

  $nonCompliance = ""

  foreach ($hostObject in $this.HostArray)
  {
    if ($hostObject.hostIssues)
    {
      $nonCompliance += $hostObject.gatherAnalysisHTML()
    }
  }

  foreach ($VMObject in $this.VMArray)
  {
    if ($VMObject.VMIssues)
    {
      $nonCompliance += $VMObject.gatherAnalysisHTML()
    }
  }
  $nonCompliance
}

# This method and runs methods to dump info about the host and VM objects within
# the cluster.
$ParentClusterClass | Add-Member -MemberType ScriptMethod -Name "dumpParent" -Value {

  foreach ($hostObject in $this.HostArray)
  {
    $hostObject.dump()
  }

  foreach ($VMObject in $this.VMArray)
  {
    $VMObject.dump()
  }
}

# Array of the networks we expect on each host (i.e. not guest networks)
$hostNetworks = @("vMotion", "vCenter Management Network", "Management Network")

################################################################################
# Class ClusterClass
#
# ClusterClass defines cluster objects that hold information about the cluster
# and holds objects about all VMs and Hosts within it. It contains methods to:
#   Get all relevant info about the cluster (getInfo).
#   Analyse that info (analysis).
#   Collate that analysis so it can be used to generate an HTML analysis
#     document (gatherAnalysisHTML and gatherAnalysisTable).
#   Output all that information in a set of files (dump).
#
# ClusterClass is a child class of ParentClusterClass.
#
# Each cluster object contains two arrays which contain the VM objects and host
# objects within that cluster. When the getInfo, analysis, gatherAnalysisHTML or
# dump methods are run on a cluster the method runs the analogous methods on each
# VM and host within that cluster (using the "parent" version of the method).
################################################################################
$ClusterClass = $ParentClusterclass.psobject.copy()

# The following are fields for data we want to output.
$ClusterClass | Add-Member -type NoteProperty -name name -value $null
$ClusterClass | Add-Member -type NoteProperty -name HAObject -value $null
$ClusterClass | Add-Member -type NoteProperty -name HAenabled -value $null
$ClusterClass | Add-Member -type NoteProperty -name IsolationResponse -value $null
$ClusterClass | Add-Member -type NoteProperty -name DASconfig -value $null
$ClusterClass | Add-Member -type NoteProperty -name VMCPconfig -value $null
$ClusterClass | Add-Member -type NoteProperty -name EVCMode -value $null
$ClusterClass | Add-Member -type NoteProperty -name DRSenabled -value $FALSE
$ClusterClass | Add-Member -type NoteProperty -name DRSrules -value $null
$ClusterClass | Add-Member -type NoteProperty -name DRSrulesString -value "NONE"
$ClusterClass | Add-Member -type NoteProperty -name VMAntiAffinity -value ""
$ClusterClass | Add-Member -type NoteProperty -name VMsMissingAA -value "NONE"
$ClusterClass | Add-Member -type NoteProperty -name SRIOV -value $FALSE
$ClusterClass | Add-Member -type NoteProperty -name iSCSI -value $FALSE
$ClusterClass | Add-Member -type NoteProperty -name TriggeredAlarms -value $null
$ClusterClass | Add-Member -type NoteProperty -name ConfigIssues -value $null

 # The following fields hold analysis of above fields.
$ClusterClass | Add-Member -type NoteProperty -name VMAntiAffinityIssue -value $FALSE
$ClusterClass | Add-Member -type NoteProperty -name HAenabledIssue -value $FALSE
$ClusterClass | Add-Member -type NoteProperty -name NetworkInconsistencyIssues -value @()
$ClusterClass | Add-Member -type NoteProperty -name FailoverHostIssues -value @()

 # The following fields hold analysis of above fields only applicable to vNow deployments.
$ClusterClass | Add-Member -type NoteProperty -name vNowEVCIssue -value $FALSE
$ClusterClass | Add-Member -type NoteProperty -name vNowvCenterIssue -value $FALSE
$ClusterClass | Add-Member -type NoteProperty -name vNowHAIssues -value @()
$ClusterClass | Add-Member -type NoteProperty -name vNowWrongSRIOVHosts -value @()
$ClusterClass | Add-Member -type NoteProperty -name vNowWrongiSCSIHosts -value @()
$ClusterClass | Add-Member -type NoteProperty -name vNowWrongSubnet -value $FALSE
$ClusterClass | Add-Member -type NoteProperty -name vNowAlarmed -value $FALSE

# When getInfo is called on a new ClusterObject it takes a cluster,
# interrogates it and populates the ClusterObject with info about the cluster.
# It then takes each host and VM within the cluster and creates objects
# populated with info about each host and VM.
$ClusterClass | Add-Member -MemberType ScriptMethod -Name "getInfo" -Value {
  param(
    [Parameter(Mandatory=$TRUE)]
    [object]$cluster
  )
  $this.name = $cluster.name
  $this.cluster = $cluster

  write-host "Collecting info for cluster $($this.name)"

  # This gets infomation relating to the HA settings on the cluster and saves
  # it in the property HAString.
  $this.HAenabled = $cluster.HAenabled
  if ($this.HAenabled -eq $TRUE)
  {
    $clusterView = Get-View $cluster
    $this.DASconfig = $clusterView.configuration.DASconfig
    $this.VMCPconfig = $this.DASconfig.defaultVMSettings.VMComponentProtectionSettings

    if ($cluster.HAadmissionControlEnabled -eq $TRUE)
    {
      $clusterAdmisCtrlInfo = $clusterView.Summary.admissionControlInfo
      $failoverHosts = $clusterAdmisCtrlInfo.hostStatus
      $AdmisCtrlPolicy = $this.DASconfig.admissionControlPolicy

      if ($failoverHosts)
      {
        $this.HAObject = HostHACreate($failoverHosts)
      }
      elseif ($clusterAdmisCtrlInfo.currentFailoverLevel)
      {
        $this.HAObject = SlotHACreate($AdmisCtrlPolicy)
      }
      elseif ($clusterAdmisCtrlInfo.currentCpuFailoverResourcesPercent)
      {
        $this.HAObject = ResourceHACreate($AdmisCtrlPolicy)
      }
      else
      {
        $this.HAObject = InvalidHACreate
      }
    }
  }
  $this.IsolationResponse = $cluster.HAIsolationResponse
  $this.EVCMode = $cluster.EVCMode

  # Information about the DRS rules on the cluster is got.
  $this.DRSenabled = $cluster.DRSenabled
  $this.DRSrules = $cluster | Get-DRSRule

  # An array is created within the clusterObject to hold all the hostObjects
  # Each hostObject is populated with information we want.
  $VMHosts = $cluster | get-VMhost
  if ($VMHosts)
  {
    foreach ($VMHost in $VMHosts)
    {
      $this.HostArray += $(CreateHostObject($VMHost))
    }
    $script:globalHostList += $this.HostArray
  }

  # An array is created within the clusterObject to hold all the VMObjects
  # Each VMObject is populated with information we want. Different VMObjects
  # are created depending on the type of VM (e.g. Perimeta, CFS, etc).
  if (!$vmName)
  {
    $VMs = $cluster | get-VM
  }
  else
  {
    $VMs = $cluster | get-VM -Name "$vmName"
  }

  if ($VMs)
  {
    # createVMObjects is used to get the VMware VM and create a VM object that
    # holds the information about the VM.
    $this.VMArray = createVMObjects($VMs)
    $script:globalVMList += $this.VMArray
  }

  # Get alarm information for cluster.
  $clusterView = Get-View $cluster
  $this.TriggeredAlarms = $clusterView.TriggeredAlarmState
  $this.ConfigIssues = $clusterView.ConfigIssue

  if ($vNOW)
  {
    # Assume that we either have all hosts with iSCSI or none, as should be the
    # case for vNOW.  We'll check this later - ditto for SR-IOV
    $this.SRIOV = $this.HostArray[0].SRIOV
    $this.iSCSI = $this.HostArray[0].iSCSI

    # For vNOW the vCenter should be a VM on the only (non-SRIOV) cluster, and
    # it should be called vcsa-<site name>, and have one IPv4 address.  Get it
    # for future use.  If there's multiple IP addresses, assume it's the first.
    $vcsa = get-vm | ? {($_.Name -like "vcsa-*")}
    if ($vcsa.count -eq 1)
    {
      $IPs = $vcsa.guest.IPAddress | where {([IPAddress]$_).AddressFamily -eq [System.Net.Sockets.AddressFamily]::InterNetwork}
      if ($IPs.count -eq 1)
      {
        $script:vCenterIP = $IPs
      }
      else
      {
        $script:vCenterIP = $IPs[0]
      }
    }
  }
}

# analysis analyses particular properties of the cluster object to see if they
# are compliant with the VMware Deployment Design guide and runs a method to do
# the same on VM and host objects in the cluster.
$ClusterClass | Add-Member -MemberType ScriptMethod -Name "analysis" -Value {

  # analysisParent is called to call analysis methods on the host and VM
  # objects.
  $this.analysisParent()

  # Analyse the DRS settings on the cluster.
  $this.VMAntiAffinity = "no partnered VMs present"
  $this.VMsMissingAA = "NONE"

  # All VMs that operate in pairs need anti-affinity DRS rules
  $pairedVmIds = @()
  $VMs = $cluster | get-VM
  foreach ($VM in $VMs)
  {
    if (IsPairedProduct($VM))
    {
      $pairedVmIds += $VM.id
    }
  }

  if ($pairedVmIds)
  {
    if ($this.DRSrules)
    {
      # Only certain anti-affinity rules should be used for paired CFSs, so
      # inappropriate DRS rules are removed.
      $validDRSrules = validateDRSrules($this.DRSrules)

      # VMs with valid DRS rules are compiled into a single list.
      $VMIdlist = @()

      foreach ($validDRSrule in $validDRSrules)
      {
        foreach($VMId in $validDRSrule.VMIds)
        {
          $VMIdlist += $VMId
        }
      }

      # If a paired VM is not listed as having a valid DRS rule associated
      # with it, then we add it to a list of paired VMs without anti-affinity.
      # A previous method relied on removing an item from $pairedVmIds, but this
      # remove method is not offered by PowerCLI 2.0 so this was changed.
      $nonDrsPairedVMs = @()

      foreach ($pairedVmId in $pairedVmIds)
      {
        $hasDrsRule = $FALSE
        foreach ($VMId in $VMIdlist)
        {
          if ($VMId -eq $pairedVmId)
          {
            $hasDrsRule = $TRUE
            break
          }
        }
        if (!$hasDrsRule)
        {
          $nonDrsPairedVMs += $pairedVmId
        }
      }

      if ($nonDrsPairedVMs.length -ge 1)
      {
        $this.VMAntiAffinity =
                       "missing for $($nonDrsPairedVMs.length) partnered VM(s)"
      }
      else
      {
        $this.VMAntiAffinity = "present for all partnered VMs"
      }
    }
    else
    {
      $this.VMAntiAffinity = "none configured"
      $nonDrsPairedVMs = $pairedVmIds
    }

    if ($nonDrsPairedVMs.length -ge 1)
    {
      $nonDrsPairedVmNames = @()
      foreach ($nonDrsVmId in $nonDrsPairedVMs)
      {
        $VM = get-VM -id "$nonDrsVmId"
        $nonDrsPairedVmNames += $VM.name
      }
      $this.VMsMissingAA = ($nonDrsPairedVmNames | sort) -Join ", "

      if ($this.DRSenabled -and $this.nonPerimetaVMsPresent)
      {
        $this.VMAntiAffinityIssue = $TRUE
        $this.clusterIssues += 1
      }
    }

    if ($vNow)
    {
      if ($this.TriggeredAlarms -or $this.ConfigIssues)
      {
        $this.vNowAlarmed = $TRUE
        $this.clusterIssues += 1
      }
    }
  }

  # Check that any non-distributed port groups configured on the hosts are
  # consistent, i.e. they have the same VLAN IDs.  They'll already have been checked
  # to be correct at the host level, i.e. correct NIC teaming etc.
  # Do this by checking each host has the same networks, then checking each host
  # matches the first host.
  $allHostNetworks = $this.HostArray | select -expandproperty VMNetworks | select -expandproperty name | sort-object | get-unique
  $firstHostNetworks = $this.HostArray[0].VMNetworks
  $firstHost = $this.HostArray[0].name
  foreach ($hostObject in $this.HostArray)
  {
    $networkNames = $hostObject.VMNetworks | select -expandproperty name
    $missing = $allHostNetworks | ? { $networkNames -NotContains $_}

    if ($missing)
    {
      $this.NetworkInconsistencyIssues += "$($hostObject.name) missing network(s): $missing"
      $this.clusterIssues += 1
    }

    # We don't check cross host consistency for the networks in $hostNetworks
    # because we can explicitly check each of these is correct at the host
    # level (unlike guest networks where there is no 'correct' configuration).
    foreach ($network in $hostObject.VMNetworks | ? {($hostNetworks -notcontains $network.name)})
    {
      $firstHostNet = $firstHostNetworks | ? {($_.name -eq $network.name)}
      if ($firstHostNet)
      {
        if ($network.vlanid -ne $firstHostNet.vlanid)
        {
          $this.NetworkInconsistencyIssues += "Inconsistent VLAN ID for $($network.name) on $($hostObject.name)"
          $this.clusterIssues += 1
        }

        $NICs = Get-NicTeamingPolicy $network | select -expandproperty ActiveNic
        $firstHostNICs = Get-NicTeamingPolicy $firstHostNet | select -expandproperty ActiveNic
        if (!$NICs)
        {
          $this.NetworkInconsistencyIssues += "No active NICs configured for $($network.name) on $($hostObject.name)"
          $this.clusterIssues += 1
        }
        elseif (($firstHostNICs) -and (compare-object $NICs $firstHostNICs))
        {
          $this.NetworkInconsistencyIssues += "Inconsistent NIC configuration for $($network.name): `
            $firstHost is using $firstHostNICs, $($hostObject.name) is using $NICs"
          $this.clusterIssues += 1
        }
      }
    }
  }

  # analyse HA settings on cluster
  # Note that vNOW SR-IOV and vSphere Essentials clusters should not have HA enabled.
  if (($this.HAenabled -ne $TRUE) -and !($vNOW -and $this.SRIOV) -and !($script:NoVSphereHA))
  {
    $this.HAenabledIssue = $TRUE
    $this.clusterIssues += 1
  }

  # Check failover hosts are all green
  foreach ($failoverHost in $this.HAObject.failoverObjects)
  {
    if ($failoverHost.Status -ne "green")
    {
      $this.FailoverHostIssues += "Failover host $($failoverHost.Host) has status $($failoverHost.Status)"
      $this.clusterIssues += 1
    }
  }

  # Perform vNOW checking
  if ($vnow)
  {
    # It's an error if we failed to work out the vCenter IP address before
    if (!$script:vCenterIP)
    {
      $this.vNowvCenterIssue = $TRUE
      $this.clusterIssues += 1
    }

    # Check HA settings - note that SR-IOV cluster must have HA disabled for vNOW
    if ($this.SRIOV)
    {
      if ($this.HAenabled -eq $TRUE)
      {
        $this.vNowHAIssues += "VMware HA should be disabled on SR-IOV cluster"
        $this.clusterIssues += 1
      }
    }
    else
    {
      # Only supported vNOW admission control policy is a single dedicated
      # failover host.
      if (@($this.HAObject.failoverObjects).length -ne 1)
      {
        $this.vNowHAIssues += "Not configured to use a single dedicated failover host"
        $this.clusterIssues += 1
      }

      if ($this.IsolationResponse -ne "DoNothing")
      {
        $this.vNowHAIssues += "Host isolation response is $($this.IsolationResponse), it should be DoNothing."
        $this.clusterIssues += 1
      }

      if ($this.DASConfig.HostMonitoring -ne "enabled")
      {
        $this.vNowHAIssues += "Host monitoring not enabled"
        $this.clusterIssues += 1
      }

      if ($this.DASConfig.VmMonitoring -ne "vmAndAppMonitoring")
      {
        $this.vNowHAIssues += "VM and Application Monitoring not enabled"
        $this.clusterIssues += 1
      }

      if ($this.DASConfig.HBDatastoreCandidatePolicy -ne "allFeasibleDs")
      {
        $this.vNowHAIssues += "Datastore Heartbeating should be set to Automatically select datastores"
        $this.clusterIssues += 1
      }

      # APD and PDL should be configured
      if ($this.VMCPconfig.VmStorageProtectionForAPD -ne "restartConservative")
      {
        $this.vNowHAIssues += "Response for Datastore with All Paths Down (APD) `
          is $($this.VMCPconfig.VmStorageProtectionForAPD), it should be restartConservative"
        $this.clusterIssues += 1
      }
      if ($this.VMCPconfig.VmStorageProtectionForPDL -ne "restartAggressive")
      {
        $this.vNowHAIssues += "Response for Datastore with Permanent Device Loss (PDL) is `
          $($this.VMCPconfig.VmStorageProtectionForPDL), it should be restartAggressive"
        $this.clusterIssues += 1
      }

      # vNOW should be configured with 2 isolation addresses, not using default
      foreach ($option in $this.DASConfig.Option)
      {
        if ($option.key -eq "das.isolationaddress0")
        {
          $d0 = $option.value
        }
        elseif ($option.key -eq "das.isolationaddress1")
        {
          $d1 = $option.value
        }
        elseif ($option.key -eq "das.usedefaultisolationaddress")
        {
          $d2 = $option.value
        }
        else
        {
          $this.vNowHAIssues += "$($this.name) has unsupported advanced option $option.key"
          $this.clusterIssues += 1
        }
      }

      if ((!$d0) -or (!$d1))
      {
        $this.vNowHAIssues += "$($this.name) doesn't have two isolation addresses defined"
        $this.clusterIssues += 1
      }

      if ($d2 -ne "false")
      {
        $this.vNowHAIssues += "$($this.name) doesn't have default isolation address turned off"
        $this.clusterIssues += 1
      }
    }

    if (($this.EVCMode -ne "intel-haswell") -and ($this.EVCMode -ne "intel-broadwell"))
    {
      $this.vNowEVCIssue = $TRUE
      $this.clusterIssues += 1
    }

    # Check for consistency in whether SR-IOV and iSCSI is configured on each host
    foreach ($VMHost in $VMHosts)
    {
      if ($VMHost.SRIOV -ne $this.SRIOV)
      {
        $this.vNowWrongSRIOVHosts = $TRUE
        break
      }
    }

    foreach ($VMHost in $VMHosts)
    {
      if ($VMHost.iSCSI -ne $this.iSCSI)
      {
        $this.vNowWrongiSCSIHosts = $TRUE
        break
      }
    }

    # Check that for the subnets for every VMKernel adapter are
    # consistent for hosts in this cluster.
    # This is done by matching the subnets to the first host in the cluster.
    $firstHost = $this.HostArray[0]
    $firstHostVMKs = $firstHost.VMKernelAdaptors

    foreach ($otherVMHost in $this.HostArray | select -skip 1)
    {
      $otherHostVMKs = $otherVMHost.VMKernelAdaptors

      foreach ($VMKernel in $otherHostVMKs)
      {
        $firstHostVMKernel = $firstHostVMKs | ? {$_.PortGroupName -eq $VMKernel.PortGroupName}

        if ($VMKernel.SubnetMask -ne $firstHostVMKernel.SubnetMask)
        {
          $this.vNowWrongSubnet = $TRUE
          break
        }
      }
    }

    if ($this.vNowWrongSubnet -eq $TRUE)
    {
      $this.clusterIssues += 1
    }
  }
}

# gatherAnalysisHTML returns a HTML string of all non compliant settings that
# have been detected on a cluster and runs methods to do the same on host and
# VM objects.
$ClusterClass | Add-Member -MemberType ScriptMethod -Name "gatherAnalysisHTML" -Value {

  $nonComplianceReport = ""
  $nonComplianceReport += "<H2>Cluster $($this.name)</H2>"

  if ($this.HAenabledIssue -eq $TRUE)
  {
    $nonComplianceReport += "<div class=`"tab`">HA is not enabled</div>"
    $nonComplianceReport += "<div class=`"tab`">HA should be enabled with `
      admission control set to: dedicated failover hosts</div>"
    $nonComplianceReport += "<div class=`"tab`">Please refer to the `
      `"VMware HA Clusters`" section of the VMware Deployment Design Guide `
      for more information</div> <BR>"
  }

  if ($this.vNowvCenterIssue -eq $TRUE)
  {
    $nonComplianceReport += "<div class=`"tab`">Failed to find vCenter on the cluster</div> <BR>"
  }

  if ($this.vNowHAIssues)
  {
    $nonComplianceReport += "<div class=`"tab`">Issue(s) found with HA configuration:</div>"
    foreach ($issue in $this.vNowHAIssues)
    {
      $nonComplianceReport += "<div class=`"tab`">$issue</div>"
    }
    $nonComplianceReport += "<BR>"
  }

  if ($this.VMAntiAffinityIssue -eq $TRUE)
  {
    $nonComplianceReport += "<div class=`"tab`">There are partnered VMs without `
      Anti-affinity DRS Rules</div>"
    $nonComplianceReport += "<div class=`"tab`">The VMs without Anti-affinity `
      DRS Rules are: $($this.VMsMissingAA)</div>"
    $nonComplianceReport += "<div class=`"tab`">Please refer to the `
      `"Distribution of VMs across hosts`" section of the VMware Deployment `
      Design Guide for more information</div> <BR>"
  }

  if ($this.NetworkInconsistencyIssues )
  {
    $nonComplianceReport += "<div class=`"tab`">Inconsistencies found with cluster networking:</div>"
    foreach ($issue in $this.NetworkInconsistencyIssues)
    {
      $nonComplianceReport += "<div class=`"tab`">$issue</div>"
    }

    $nonComplianceReport += "<div class=`"tab`">WARNING: If resolving any of these issues requires renaming a network `
       you must restart the vCenter and any VMs using the affected networks afterwards.</div><BR>"
  }

  if ($this.FailoverHostIssues)
  {
    $nonComplianceReport += "<div class=`"tab`">Issues found with failover hosts:</div>"
    foreach ($issue in $this.FailoverHostIssues)
    {
      $nonComplianceReport += "<div class=`"tab`">$issue</div>"
    }
    $nonComplianceReport += "</div><BR>"
  }

  if ($this.vNowWrongSRIOVHosts)
  {
    $nonComplianceReport += "<div class=`"tab`">Inconsistencies in host SR-IOV configuration:</div>"
    $nonComplianceReport += "<div class=`"tab`">Hosts with SR-IOV:"
    $this.VMHosts | ? {$_.SRIOV} | ForEach-Object { $nonComplianceReport += $_.Name }
    $nonComplianceReport += "</div><div class=`"tab`">Hosts without SR-IOV:"
    $this.VMHosts | ? {!$_.SRIOV} | ForEach-Object { $nonComplianceReport += $_.Name }
    $nonComplianceReport += "</div><BR>"
  }

  if ($this.vNowWrongiSCSIHosts)
  {
    $nonComplianceReport += "<div class=`"tab`">Inconsistencies in host iSCSI configuration:</div>"
    $nonComplianceReport += "<div class=`"tab`">Hosts with iSCSI:"
    $this.VMHosts | ? {$_.iSCSI} | ForEach-Object { $nonComplianceReport += $_.Name }
    $nonComplianceReport += "</div><div class=`"tab`">Hosts without iSCSI:"
    $this.VMHosts | ? {!$_.iSCSI} | ForEach-Object { $nonComplianceReport += $_.Name }
    $nonComplianceReport += "</div><BR>"
  }

  if ($this.vNowEVCIssue -eq $TRUE)
  {
    $nonComplianceReport += "<div class=`"tab`">Cluster has EVC mode $EVCMode</div>"
    $nonComplianceReport += "<div class=`"tab`">It should be configured with EVC mode Intel Haswell or Broadwell</div> <BR>"
  }

  if ($this.vNowWrongSubnet -eq $TRUE)
  {
    $nonComplianceReport += "<div class=`"tab`"> Subnet masks are inconsistent among hosts within this cluster. <BR>"

    foreach ($VMKernel in $this.HostArray[0].VMKernelAdaptors)
    {
      $nonComplianceReport += "For VMKernel Adapter $($VMKernel.PortGroupName):"
      $nonComplianceReport += "<ul>"

      foreach ($VMHost in $this.HostArray)
      {
        $hostVMKs = $VMHost.VMKernelAdaptors | ? {$_.PortGroupName -eq $VMKernel.PortGroupName}
        $nonComplianceReport += "<li> Host $($VMHost.Name) has subnet mask: $($hostVMKs.SubnetMask) </li>"
      }

      $nonComplianceReport += "</ul>"
    }

    $nonComplianceReport += "</div><BR>"
  }

  if ($this.vNowAlarmed)
  {
    $nonComplianceReport += "<div class=`"doubletab`">"

    $nonComplianceReport += "There are the following alarms on the cluster"
    $nonComplianceReport += "<ul>"

    foreach ($alarm in $this.TriggeredAlarms)
    {
      $alarmDef = Get-View -Id $alarm.Alarm

      #vNOW accepts "Virtual Machine Memory usage as a benign alarm"
      if ($alarmDef.Info.Name -like "*Virtual Machine Memory Usage*")
      {
        continue
      }

      $nonComplianceReport += "<li>$($alarmDef.Info.Name)</li>"
    }

    foreach ($cfgIssue in $this.ConfigIssues)
    {
      $nonComplianceReport += "<li>$($cfgIssue.FullFormattedMessage)</li>"
    }

    $nonComplianceReport += "</ul></div><BR>"
  }

  $nonComplianceReport += $this.gatherAnalysisHTMLParent()

  $nonComplianceReport
}

# gatherAnalysisTable returns an array of information of analysis data. Each
# item in the array lists the setting name, the setting value, and whether
# the setting is compliant with the VDDG.
$ClusterClass | Add-Member -MemberType ScriptMethod -Name "gatherAnalysisTable" -Value {

  if ($this.HAObject)
  {
    $HAsetting = $this.HAObject.HAsetting
  }
  elseif ($this.HAenabled -eq $TRUE)
  {
    $HAsetting = "HA is enabled with no admission control"
  }
  else
  {
    $HAsetting = "HA is not enabled"
  }

  $clusterTableRow = @(@("Cluster", $this.Name, $FALSE), `
    @("DRSenabled", $this.DRSenabled, $FALSE), `
    @("VMAnti-Affinity rules", $this.VMAntiAffinity, $this.VMAntiAffinityIssue), `
    @("Partnered VMs without Antiaffinity", $this.VMsMissingAA, $this.VMAntiAffinityIssue), `
    @("HAenabled", $this.HAenabled, $this.HAenabledIssue), `
    @("Admission Control Setting", $HAsetting, $FALSE), `
    @("has non-Perimeta guests", $this.nonPerimetaVMsPresent, $FALSE))

  $clusterTableRow
}

# This method dumps info about the cluster to a txt file and runs a method to do
# the same on host and VM objects within the cluster.
$ClusterClass | Add-Member -MemberType ScriptMethod -Name "dump" -Value {

  $DRSrulesString = "None"
  if ($this.DRSrules)
  {
    $DRSrulesString = ""
    foreach ($drs_rule in $this.DRSrules)
    {
      if ($drs_rule.KeepTogether)
      {
        $DRSrulesString += "Affinity VM rule: "
      }
      else
      {
        $DRSrulesString += "Anti-affinity VM rule: "
      }

      foreach ($vm_id in $drs_rule.VMIds)
      {
        $drs_vm = Get-VM -Server $viServer -Id "$vm_id"
        $DRSrulesString += "$drs_vm "
      }
      $DRSrulesString += " `r`n"
    }
  }

  write-host "Dumping info for cluster $($this.name)..."

  $filename = "$destDir/cluster_$($this.name)_details.txt"
  "Cluster Config:" | Out-File -filepath $filename -Append -Encoding ASCII
  $this.cluster | select * | Ft -autosize | out-string -width 4096 | Out-File -filepath $filename -Append -Encoding ASCII

  "HA Settings:" | Out-File -filepath $filename -Append -Encoding ASCII

  if ($this.HAObject)
  {
    $this.HAObject.dump() | Ft -autosize | out-string -width 4096 | Out-File -filepath $filename -Append -Encoding ASCII
  }
  elseif ($this.HAenabled -eq $TRUE)
  {
    "HA is enabled with no admission control" | Out-File -filepath $filename -Append -Encoding ASCII
  }
  else
  {
    "HA is not enabled" | Out-File -filepath $filename -Append -Encoding ASCII
  }

  "DRS Rules:" | Out-File -filepath $filename -Append -Encoding ASCII
  if ($DRSrulesString -ne "None")
  {
    $DRSrulesString | Ft -autosize | out-string -width 4096 | Out-File -filepath $filename -Append -Encoding ASCII
  }
  else
  {
    "No DRS Rules detected for $($this.name)" | Out-File -filepath $filename -Append -Encoding ASCII
  }

  # dumpParent is called to run dump methods on the host and VM objects.
  $this.dumpParent()
}

################################################################################
# Class GhostClusterClass
#
# GhostClusterClass is a child class of ParentClusterClass. It is designed to
# only hold information for VMs and hosts not contained in any cluster.
#
# It has methods that act as a common interface that runs functional methods in
# ParentClusterClass, as well as a method for creating VM/host objects from
# VMs/hosts contained in an object it is passed ($unclusteredHosts).
################################################################################
$GhostClusterClass = $ParentClusterclass.psobject.copy()

# When getInfo is called on a GhostClusterObject it takes the hosts in the
# hostArray and creates objects populated with info about each host and VM on
# the hosts.
$GhostClusterClass | Add-Member -MemberType ScriptMethod -Name "getInfo" -Value {
  param(
    [Parameter(Mandatory=$TRUE)]
    [object]$unclusteredHosts
  )
  $this.cluster = $unclusteredHosts

  write-host "Collecting info for hosts outside of clusters"

  $VMhosts = $this.cluster.hosts
  $VMs = @()
  # For each host in the array a VMhost object is generated and the information
  # about the host stored in the object. If a VM has been specified then a VM
  # object will be created for it and information about the VM stored in the VM
  # object. If no VM is specified then all VMs on the host will have VM objects
  # created for them.
  foreach ($VMHost in $VMHosts)
  {
    $this.HostArray += $(CreateHostObject($VMHost))

    if (!$vmName)
    {
      $VMs += $VMhost | get-VM
    }
    else
    {
      $VMs += $VMhost | get-VM -Name "$vmName"
    }
  }
  if ($VMs)
  {
    # createVMObjects is used to get the VMware VM and create a VM object that
    # holds the information about the VM.
    $this.VMArray = createVMObjects($VMs)
    $script:globalVMList += $this.VMArray
  }
  $script:globalHostList += $this.HostArray
}

# analysis analyses particular properties of the cluster object to see if they
# are compliant with the VMware Deployment Design guide and runs methods to do
# the same on VM and host objects in the cluster.
$GhostClusterClass | Add-Member -MemberType ScriptMethod -Name "analysis" -Value {

  $this.analysisParent()
}

# gatherAnalysisHTML runs methods that return an HTML string of all non
# compliant settings that have been detected on hosts and VM objects.
$GhostClusterClass | Add-Member -MemberType ScriptMethod -Name "gatherAnalysisHTML" -Value {

  $this.gatherAnalysisHTMLParent()
}

# gatherAnalysisTable returns an array stating there are unclustered hosts.
$GhostClusterClass | Add-Member -MemberType ScriptMethod -Name "gatherAnalysisTable" -Value {

  $clusterTableRow = @(@(),@())

  $clusterTableRow
}

# This method runs methods to dump info about the hosts and VM objects within
# the ghost cluster.
$GhostClusterClass | Add-Member -MemberType ScriptMethod -Name "dump" -Value {

  $this.dumpParent()
}

################################################################################
# Class HAClass
#
# HAClass contains the HA information for a cluster It contains methods to:
#   Get all relevant admission control info info about the cluster (getInfo).
#   Return relevant admission control output  (dump).
#
# Subclasses are generated for the 4 different admission control settings which
# specify the specifics of each method while retaining a common interface. The
# subclasses are:
#   HostHAClass - HA with dedicated reserve failover hosts
#   SlotHAClass - HA based on the number of slots available on hosts.
#   ResourceHAClass - HA based on a percentage of resources available on hosts.
#   InvalidHAClass - HA with invalid settings (e.g. having one host in a cluster).
################################################################################
$HAClass = new-object psobject -Property @{
  HAsetting = $null

  failoverHosts = $null
  failoverObjects = @()

  clusterFailoverLevel = $null
  clusterSlotCPU = $null
  clusterSlotMem = $null

  cpuFailoverResPer = $null
  memoryFailoverResPer = $null
}

# HostHAClass describes an object used for storing info about clusters using
# failover hosts as their admission control setting.
$HostHAClass = $HAClass.psobject.copy()
$HostHAClass.HAsetting += "Admission Control Setting: Reserve failover hosts `r`n"

# getInfo takes the failover hosts of a cluster and populates the HostHAObject
# with the name and status of failover Hosts
$HostHAClass | Add-Member -MemberType ScriptMethod -Name "getInfo" -Value {
  param(
    [Parameter(Mandatory=$TRUE)]
    [object]$failoverHosts
  )
  $this.failoverObjects = @()

  foreach ($failoverHost in $failoverHosts)
  {
    $failoverObject = "" | Select-Object Host, Status
    $failoverObject.Status = $failoverHost.status
    $gotHost = get-VMhost -id $failoverHost.host
    $failoverObject.Host = $gotHost.Name
    $this.failoverObjects += $failoverObject
  }
}

# dump returns gathered information about the failover hosts.
$HostHAClass | Add-Member -MemberType ScriptMethod -Name "dump" -Value {
  $HAoutput = "$($this.HAsetting)"
  $HAoutput += "$($this.failoverObjects)"
  $HAoutput
}

# HostHAClass describes an object used for storing info about clusters using
# slots as their admission control setting.
$SlotHAClass = $HAClass.psobject.copy()
$SlotHAClass.HAsetting = "Admission Control Setting: Reserve Failover slots `r`n"

# getInfo takes the admission control policy of a cluster and populates the
# object with info about the slot size and failover level.
$SlotHAClass | Add-Member -MemberType ScriptMethod -Name "getInfo" -Value {
  param(
    [Parameter(Mandatory=$TRUE)]
    [object]$AdmisCtrlPolicy
  )
  $this.clusterFailoverLevel = $AdmisCtrlPolicy.failoverLevel
  $this.clusterSlotCPU = $AdmisCtrlPolicy.slotpolicy.cpu
  $this.clusterSlotMem = $AdmisCtrlPolicy.slotpolicy.memory
}

# dump returns a string of information about the admission control settings.
$SlotHAClass | Add-Member -MemberType ScriptMethod -Name "dump" -Value {
  $HAoutput = "$($this.HAsetting)"
  $HAoutput += "Cluster Failover Level: $($this.clusterFailoverLevel) `r`n"
  if ($this.clusterSlotMem)
  {
    $HAoutput += "Slot memory size: $($this.clusterSlotMem) MB `r`n"
    $HAoutput += "Slot CPU size: $($this.clusterSlotCPU) MHz `r`n"
  }
  else
  {
    $HAoutput += "Slot size equals the largest VM reservations. `r`n"
  }
  $HAoutput
}

# HostHAClass describes an object used for storing info about clusters using
# dedicated resources as their admission control setting.
$ResourceHAClass = $HAClass.psobject.copy()
$ResourceHAClass.HAsetting += "Admission Control Setting: Failover resources `r`n"

# getInfo takes the admission control policy of the cluster and populates the
# object with info about the dedicated resource percentage.
$ResourceHAClass | Add-Member -MemberType ScriptMethod -Name "getInfo" -Value {
  param(
    [Parameter(Mandatory=$TRUE)]
    [object]$AdmisCtrlPolicy
  )
  $this.cpuFailoverResPer = $AdmisCtrlPolicy.cpuFailoverResPer
  $this.memoryFailoverResPer = $AdmisCtrlPolicy.memoryFailoverResPer
}

# dump returns a string of information about the admission control settings.
$ResourceHAClass | Add-Member -MemberType ScriptMethod -Name "dump" -Value {
  $HAoutput = "$($this.HAsetting)"
  $HAoutput += "Failover percentage of CPU resources:`
    $($this.cpuFailoverResPer) `r`n"
  $HAoutput += "Failover percentage of Memory resources:`
    $($this.memoryFailoverResPer) `r`n"
  $HAoutput
}

# HostHAClass describes an object used for storing info about clusters using
# dedicated resources as their admission control setting.
$InvalidHAClass = $HAClass.psobject.copy()
$InvalidHAClass.HAsetting = "Admission control settings are not valid. `r`n `r`n"

# dump returns a string of information about the admission control settings.
$InvalidHAClass | Add-Member -MemberType ScriptMethod -Name "dump" -Value {
  $this.HAsetting
}

################################################################################
# Class HostClass
#
# HostClass defines host objects that hold information about the host. It
# contains methods to:
#   Get all relevant info about the host (getInfo).
#   Analyse that info (analysis).
#   Collate that analysis so it can be used to generate an HTML analysis
#     document (gatherAnalysisHTML and gatherAnalysisTable).
#   Output all that information in a set of files (dump).
################################################################################
$HostClass = new-object psobject -Property @{
  # The following are fields for data we want to output.
  name = $null
  Cluster = $null
  ConnectionState = $null
  NumCpuThreads = $null
  NumCpuSockets = $null
  NumCpuCores = $null
  ProvCpuCores = $null
  RAM = $null
  ProvRAM = $null
  HyperthreadAvailable = $null
  HyperthreadActive = $null
  ESXiVersion = $null
  NTPServer = $null
  NTPService = $null
  LockdownMode = $null
  SyslogHost = $null
  SyslogFirewallEnabled = $FALSE
  Luns = $null
  iSCSI = $null
  AccountUnlockTime = $null
  AccountLockFailures = $null
  ESXiShellIntTimeout = $null
  ESXiShellTimeout = $null
  BlockGuestBDPU = $null

  CPUpackageinfo = $null
  SystemInfo = $null
  Datastores = $null
  Network = $null
  dvSwitches = $null
  dVMnetworks = $null
  vSwitches = $null
  VMNetworks = $null
  VMKernelAdaptors = $null
  PCIdevices = $null
  NetworkDrivers = @()
  DumpCollector = $null
  SRIOV = $FALSE
  VMHostStartPolicy = $null
  PhysicalAdapters = $null

  TriggeredAlarms = $null
  ConfigIssues = $null

  # The following fields hold analysis of above fields
  HTavailableIssue = $FALSE
  HTactiveIssue = $FALSE
  numCPUthreadsIssue = $FALSE
  hostCPUIssue = @()
  NTPIssues = @()
  PhysicalAdapterIssues = @()

  # The following fields hold analysis of above fields only applicable to vNow deployments
  vNowDumpCollectorIssues = @()
  vNowSyslogIssues = @()
  vNowESXiVersionIssue = $FALSE
  vNowLockdownModeIssue = $FALSE
  vNowMultiPathingIssue = $FALSE
  vNowNumDatastoreIssue = $FALSE
  vNowNumDatastorePathIssues = @()
  vNowNetworkIssues = @()
  vNowSRIOVIssue = $FALSE
  vNowSecurityIssues = @()
  vNowVMHostStartPolicyIssue = $FALSE
  vNowAlarmed = $FALSE

  nonPerimetaVMsPresent = $FALSE
  hostIssues = $null
}

# When getInfo is called on a new HostObject it takes a VMhost, interrogates it
# and populates the HostObject with info about the VMHost.
$HostClass | Add-Member -MemberType ScriptMethod -Name "getInfo" -Value {
  param(
    [Parameter(Mandatory=$TRUE)]
    [object]$VMHost
  )
  $this.name = $VMHost.name

  write-host "Collecting info for host $($this.name)"

  $totalProvCPU = 0
  $totalProvRAM = 0

  $VMs = $VMHost | Get-VM
  foreach ($VM in $VMs)
  {
    $totalProvCPU += $VM.NumCPU
    $totalProvRAM += $VM.MemoryGB
  }

  $hostView = $VMHost | Get-View
  $this.Cluster = $VMHost.Parent.Name
  $this.NumCpuSockets = $hostView.hardware.cpuinfo.numCpuPackages
  $this.NumCpuCores = $VMHost.NumCpu
  $this.NumCpuThreads = $hostView.hardware.cpuinfo.NumCpuThreads
  $this.ProvCpuCores = $totalProvCPU / 2
  $this.RAM = $VMHost.MemoryTotalGB
  $this.ProvRAM = $totalProvRAM
  $this.HyperthreadAvailable = $VMHost.ExtensionData.Config.HyperThread.Available
  $this.HyperthreadActive = $VMHost.HyperthreadingActive
  $this.ESXiVersion = $VMHost.version
  $this.NTPServer = $VMHost | Get-VMHostNtpServer
  $this.NTPService = $VMHost | Get-VmHostService | ? {($_.Key -eq "ntpd")}
  $this.LockdownMode = $VMHost.ExtensionData.Config.LockdownMode
  $this.SyslogHost = Get-AdvancedSetting $VMHost -name Syslog.global.logHost | select -expandproperty value
  $this.SyslogFirewallEnabled = Get-VmHostFirewallException $VMHost -name "syslog" | select -expandproperty enabled
  $this.PhysicalAdapters = $hostView.Config.Network.Pnic

  $this.SystemInfo = $hostView.Hardware.SystemInfo
  $this.Datastores = $VMHost | Get-Datastore | Get-Unique
  write-host "Datastores: $($this.Datastores)"

  $this.Luns = $VMHost | Get-VMHostHba | ? {($_.Model -ne "USB Storage Controller")} | Get-ScsiLun
  $this.iSCSI = $VMHost | Get-VMHostHba -type iSCSI

  $this.Network = $VMHost | Get-VMHostNetworkAdapter
  $this.dvSwitches = $VMHost | Get-VDSwitch
  $this.dVMnetworks = $this.dvSwitches | Get-VDPortGroup
  $this.vSwitches = $VMHost | Get-VirtualSwitch -standard
  $this.VMnetworks = $this.vSwitches | Get-VirtualPortGroup
  $this.VMKernelAdaptors = Get-VMHostNetworkAdapter -VMHost $VMHost -VMKernel
  $this.CPUpackageinfo = $hostView.Hardware.CpuPkg
  $this.PCIdevices = $hostView.Hardware.PciDevice
  $this.AccountUnlockTime = $VMHost | Get-AdvancedSetting -Name "Security.AccountUnlockTime" | select -expandproperty value
  $this.AccountLockFailures = $VMHost | Get-AdvancedSetting -Name "Security.AccountLockFailures" | select -expandproperty value
  $this.ESXiShellIntTimeout = $VMHost | Get-AdvancedSetting -Name "UserVars.ESXiShellInteractiveTimeout" | select -expandproperty value
  $this.ESXiShellTimeout = $VMHost | Get-AdvancedSetting -Name "UserVars.ESXiShellTimeout" | select -expandproperty value
  $this.BlockGuestBDPU = $VMHost | Get-AdvancedSetting -Name "Net.BlockGuestBPDU" | select -expandproperty value
  $this.TriggeredAlarms = $hostView.TriggeredAlarmState
  $this.ConfigIssues = $hostView.ConfigIssue

  # Get the ESXCLI and get info from that.
  # Note that this involves actually accessing the host, which we obviously
  # can't do if the host is unresponsive.  If so - we'll write an error, but we
  # need to make sure that not having information gathered from the ESXCLI
  # doesn't cause script failures.
  $this.ConnectionState = $VMHost.ConnectionState

  if ($this.ConnectionState -ne "NotResponding")
  {
    $esxcli = $VMHost | Get-EsxCli

    foreach ($nic in $this.Network)
    {
      if ($nic.ExtensionData.Driver)
      {
        $this.NetworkDrivers += $esxcli.network.nic.get($nic.Name).DriverInfo
      }
    }

    # Get a list of adapters that are configured for SR-IOV (e.g. vmnic7,vmnic9)
    $this.SRIOV = ($esxcli.network.sriovnic.list() | select -expandproperty Name | Sort-Object) -join ","

    $this.DumpCollector = $esxcli.system.coredump.network.get()
  }
  else
  {
    $this.NetworkDrivers += "Could not get Network Driver information - unresponsive host"
  }

  $this.VMHostStartPolicy = $VMHost | Get-VMHostStartPolicy
}

# analysis analyses particular properties of the Host object to see if they
# are compliant with the VMware Deployment Design guide.
$HostClass | Add-Member -MemberType ScriptMethod -Name "analysis" -Value {
  param(
    [Parameter(Mandatory=$TRUE)]
    [array]$VMArray
  )

  if ($this.HyperthreadAvailable -ne $TRUE)
  {
    $this.HTavailableIssue = $TRUE
    $this.hostIssues += 1
  }
  if ($this.HyperthreadActive -ne $TRUE)
  {
    $this.HTactiveIssue = $TRUE
    $this.hostIssues += 1
  }

  foreach ($VM in $VMArray)
  {
    if ($VM.host.name -eq $this.name)
    {
      if (!$VM.isPerimeta)
      {
        $this.nonPerimetaVMsPresent = $TRUE
      }
    }
  }
  if ($this.nonPerimetaVMsPresent -eq $TRUE)
  {
    if ($this.NumCpuThreads -lt ($this.ProvCpuCores + 2))
    {
      $this.numCPUthreadsIssue = $TRUE
      $this.hostIssues += 1
    }
  }

  $cpuDescriptions = @()
  foreach ($CPU in $this.CPUpackageInfo)
  {
    $cpuDescriptions += $CPU.description
  }
  $this.hostCPUIssue = $cpuDescriptions -Join ", "

  # NTP should be enabled
  if (!$this.NTPServer)
  {
    $this.NTPIssues += "No NTP server configured"
    $this.hostIssues += 1
  }

  if (($this.NTPService.Policy -ne "on") -or (!$this.NTPService.Running))
  {
    $this.NTPIssues += "NTP not configured to run automatically"
    $this.hostIssues += 1
  }

  if ($this.ConnectionState -eq "NotResponding")
  {
    $this.hostIssues += 1
  }

  # Check that for each port group, that any of the physical adapters set as
  # active or standby are not down.
  foreach ($PG in $this.VMnetworks)
  {
    $Nics = @()
    foreach ($ActiveAdapter in $PG.ExtensionData.Spec.Policy.NicTeaming.NicOrder.ActiveNic)
    {
      $Nics += $ActiveAdapter
    }

    foreach ($StandbyAdapter in $PG.ExtensionData.Spec.Policy.NicTeaming.NicOrder.StandbyNic)
    {
      $Nics += $StandbyAdapter
    }

    foreach($Nic in $Nics)
    {
      $PhysAdapter = $this.PhysicalAdapters | ? {($_.Device -eq $Nic)}

      if (!$PhysAdapter.LinkSpeed)
      {
        $this.PhysicalAdapterIssues += "Physical Adapter $Nic configured for Port Group $($PG.name) is down."
        $this.hostIssues += 1
      }
    }
  }

  # Do vNOW checking
  if ($vnow)
  {
    # For vNOW check that dump collector is enabled with the correct settings
    if (!$this.DumpCollector)
    {
      $this.vNowDumpCollectorIssues += "Could not get Dump Collector information - unresponsive host"
      $this.hostIssues += 1
    }
    elseif (!$this.DumpCollector.enabled)
    {
      $this.vNowDumpCollectorIssues += "DumpCollector not enabled"
      $this.hostIssues += 1
    }
    else
    {
      if ($this.DumpCollector.HostVNic -ne "vmk0")
      {
        $this.vNowDumpCollectorIssues += "DumpCollector not using NIC vmk0"
        $this.hostIssues += 1
      }

      if (!$this.DumpCollector.NetworkServerIP)
      {
        $this.vNowDumpCollectorIssues += "DumpCollector not configured with vCenter IP address $script:vCenterIP"
        $this.hostIssues += 1
      }
      elseif (($script:vCenterIP) -and ($this.DumpCollector.NetworkServerIP -ne $script:vCenterIP))
      {
        $this.vNowDumpCollectorIssues += "DumpCollector using $($this.DumpCollector.NetworkServerIP), not vCenter IP address $script:vCenterIP"
        $this.hostIssues += 1
      }

      if ($this.DumpCollector.NetworkServerPort -ne 6500)
      {
        $this.vNowDumpCollectorIssues += "DumpCollector not using port 6500"
        $this.hostIssues += 1
      }
    }

    if ($this.LockdownMode -ne "lockdownNormal")
    {
      $this.vNowLockdownModeIssue = $TRUE
      $this.hostIssues += 1
    }

    # vNow currently supports ESXi v6.0 and v6.5
    if (($this.ESXiVersion -ne "6.0.0") -and ($this.ESXiVersion -ne "6.5.0"))
    {
      $this.vNowESXiVersionIssue = $TRUE
      $this.hostIssues += 1
    }

    # Syslog should be configured and the firewall ports opened
    if (($script:vCenterIP) -and ($this.SyslogHost -ne "tcp://$($script:vCenterIP):514"))
    {
      $this.vNowSyslogIssues += "Configured to use $($this.SyslogHost) as syslog host - it should be tcp://$($script:vCenterIP):514"
      $this.hostIssues += 1
    }

    if  (!$this.SyslogFirewallEnabled)
    {
      $this.vNowSyslogIssues += "Firewall rules not configured to allow outbound syslog traffic"
      $this.hostIssues += 1
    }

    # Multipath should be round robin for each LUN, there should be at least two
    foreach ($lun in $this.Luns)
    {
      if ($lun.MultipathPolicy -ne "RoundRobin")
      {
        $this.vNowMultiPathingIssue = $TRUE
      }
    }

    if ($vNowMultiPathingIssue)
    {
      $this.hostIssues += 1
    }

    if ($this.Luns.count -lt 2)
    {
      $this.vNowNumDatastoreIssue = "Host doesn't have 2 or more storage Luns"
      $this.hostIssues += 1
    }

    # Direct SAS storage should have 1 active paths, 1 standby.
    # iSCSI storage should have 2 active and 2 standby
    foreach ($lun in $this.Luns)
    {
      $Paths = Get-ScsiLunPath -ScsiLun $lun | select -ExpandProperty state | group
      $NumActivePaths = $Paths | ? {($_.name -eq "Active")} | select -expandproperty Count
      $NumStandbyPaths = $Paths | ? {($_.name -eq "Standby")} | select -expandproperty Count
      if ($this.iSCSI)
      {
        if ($NumActivePaths -ne 2)
        {
          $this.vNowNumDatastorePathIssues += "Host doesn't have 2 active paths to storage device $lun"
          $this.hostIssues += 1
        }
        if ($NumStandbyPaths -ne 2)
        {
          $this.vNowNumDatastorePathIssues += "Host doesn't have 2 standby paths to storage device $lun"
          $this.hostIssues += 1
        }
      }
      else
      {
        if ($NumActivePaths -ne 1)
        {
          $this.vNowNumDatastorePathIssues += "Host doesn't have an active path to storage device $lun"
          $this.hostIssues += 1
        }
        if ($NumStandbyPaths -ne 1)
        {
          $this.vNowNumDatastorePathIssues += "Host doesn't have a standby path to storage device $lun"
          $this.hostIssues += 1
        }
      }
    }

    # There should be a single virtual (non-distributed) switch on each vNOW host
    # +1 non-distributed switch if the host is using iSCSI
    # +1 distributed switch if the host is using SR-IOV
    if ($this.SRIOV)
    {
      if ($this.dvSwitches.count -ne 1)
      {
        $this.vNowNetworkIssues += "There should be a single distributed switch configured on SR-IOV hosts"
        $this.hostIssues += 1
      }
    }
    else
    {
      if ($this.dvSwitches.count -ne 0)
      {
        $this.vNowNetworkIssues += "There should be no distributed switches configured"
        $this.hostIssues += 1
      }
    }

    if ($this.iSCSI)
    {
      if ($this.vSwitches.count -ne 2)
      {
        $this.vNowNetworkIssues += "There should be a two non-distributed switches configured on hosts with iSCSI"
        $this.hostIssues += 1
      }
    }
    else
    {
      if ($this.vSwitches.count -ne 1)
      {
        $this.vNowNetworkIssues += "There should be a single non-distributed switch configured"
        $this.hostIssues += 1
      }
    }

    # Check the NIC configuration for the hostNetworks
    # - management network and vCenter management network active 0/standby 4
    # - vMotion - active 4/standby 0
    #
    # Note we've alreadyed checked that each host's networks are the same VLAN
    # IDs at the scope of the cluster
    foreach ($netname in $hostNetworks)
    {
      $net = $this.VMnetworks | ? {($_.Name -eq $netname)}
      # skip the vCenter mangement network for SR-IOV
      if (($netname -eq "vCenter Management Network") -and ($this.SRIOV))
      {
        # skip
      }
      elseif (!$net)
      {
        $this.vNowNetworkIssues += "No network named $netname"
        $this.hostIssues += 1
      }
      else
      {
        $active = Get-NicTeamingPolicy $net | select -expandproperty activenic
        $standby = Get-NicTeamingPolicy $net | select -expandproperty standbynic

        if ($netname -eq "vMotion")
        {
          $expActive = "vmnic4"
          $expStandby = "vmnic0"
        }
        elseif ($netname -eq "Management Network")
        {
          $expActive = "vmnic0"
          $expStandby = "vmnic4"
          $ManagementVlandID = $net.vlanid
        }
        else
        {
          $expActive = "vmnic0"
          $expStandby = "vmnic4"
          $vCenterManagementVlandID = $net.vlanid
        }

        if ($active -ne $expActive)
        {
          $this.vNowNetworkIssues += "$netname should use $expActive for active NIC - it's using $active"
          $this.hostIssues += 1
        }
        if ($standby -ne $expStandby)
        {
          $this.vNowNetworkIssues += "$netname should use $expStandby for standby NIC - it's using $standby"
          $this.hostIssues += 1
        }
      }
    }

    # Check that the management VLAN ID is the same as the vCenter management
    if (($ManagementVlandID -ne $null) -and
        ($vCenterManagementVlandID -ne $null) -and
        ($ManagementVlandID -ne $vCenterManagementVlandID))
    {
      $this.vNowNetworkIssues += "Management Network is using VLAN ID $ManagementVlandID, `
        vCenter Management Network is using $vCenterManagementVlandID.  These should be the same."
      $this.hostIssues += 1
    }

    # Test security settings on vSwitch0
    $vSwitch0 = $this.vSwitches | ? {$_.Name -eq "vSwitch0"}
    if (!$vSwitch0)
    {
      $this.vNowNetworkIssues += "No vSwitch0 found."
      $this.hostIssues += 1
    }
    else
    {
      if ($vSwitch0.mtu -ne 1500)
      {
        $this.vNowNetworkIssues += "vSwitch0 has MTU $($vSwitch0.mtu), not 1500"
        $this.hostIssues += 1
      }
      if ($vSwitch0 | Get-SecurityPolicy | select -ExpandProperty MacChanges)
      {
        $this.vNowNetworkIssues += "vSwitch0 doesn't reject guest MAC address changes"
        $this.hostIssues += 1
      }
      if ($vSwitch0 | Get-SecurityPolicy | select -ExpandProperty ForgedTransmits)
      {
        $this.vNowNetworkIssues += "vSwitch0 doesn't reject guest Forged transmits"
        $this.hostIssues += 1
      }
    }

    # Check iSCSI network set up - 2 networks called iSCSI 1 and iSCSI 2 on vSwitch1
    # With MTU 9000
    if ($this.iSCSI)
    {
      $vSwitch1 = $this.vSwitches | ? {$_.Name -eq "vSwitch1"}
      if (!$vSwitch1)
      {
        $this.vNowNetworkIssues += "No vSwitch1 found for iSCSI networks."
        $this.hostIssues += 1
      }
      else
      {
        if ($vSwitch1.mtu -ne 9000)
        {
          $this.vNowNetworkIssues += "vSwitch1 has MTU $($vSwitch1.mtu), not 9000"
          $this.hostIssues += 1
        }

        $iSCSI1 = $vSwitch1 | get-VirtualPortGroup -Name "iSCSI 1"

        if (!$iSCSI1)
        {
          $this.vNowNetworkIssues += "vSwitch1 doesn't have iSCSI 1 network"
          $this.hostIssues += 1
        }
        elseif ($iSCSI1.VLANID -ne 0)
        {
          $this.vNowNetworkIssues += "iSCSI 1 network shouldn't be configured with a VLAN ID"
          $this.hostIssues += 1
        }

        $iSCSI2 = $vSwitch1 | get-VirtualPortGroup -Name "iSCSI 2"

        if (!$iSCSI2)
        {
          $this.vNowNetworkIssues += "vSwitch1 doesn't have iSCSI 2 network"
          $this.hostIssues += 1
        }
        elseif ($iSCSI2.VLANID -ne 0)
        {
          $this.vNowNetworkIssues += "iSCSI 2 network shouldn't be configured with a VLAN ID"
          $this.hostIssues += 1
        }

        $ExtraNetworks = $vSwitch1 | get-VirtualPortGroup | ? {$_.Name -ne "iSCSI 1" -and $_.name -ne "iSCSI 2"}
        if ($ExtraNetworks)
        {
          $this.vNowNetworkIssues += "iSCSI vSwitch1 has extra networks: $ExtraNetworks"
          $this.hostIssues += 1
        }
      }
    }

    # Check that guest networks have redundant active NICs and no standby
    # Note we've alreadyed checked that each host's networks are the same VLAN
    # IDs at the scope of the cluster
    foreach ($net in $this.VMnetworks | ? {($hostNetworks -notcontains $_.Name)})
    {
      $active = Get-NicTeamingPolicy $net | select -expandproperty activenic
      $standby = Get-NicTeamingPolicy $net | select -expandproperty standbynic

      if (($active.count -lt 2) -and ($net.name -notLike "iSCSI*"))
      {
        $this.vNowNetworkIssues += "Guest network $($net.name) doesn't have redundant active NICs.  It's using $active"
        $this.hostIssues += 1
      }

      if ($standby.count -ne 0)
      {
        $this.vNowNetworkIssues += "Guest network $($net.name) shouldn't have standby NICs.  It's using $standby"
        $this.hostIssues += 1
      }
    }

    # Networks should all use load balancing on source port
    foreach ($network in $this.VMnetworks)
    {
      if ((Get-NicTeamingPolicy $network | select -expandproperty loadbalancingpolicy) -ne "LoadBalanceSrcId")
      {
        $this.vNowNetworkIssues += "$($network.name) isn't loadbalancing on originating virutal port"
        $this.hostIssues += 1
      }
    }

    # There should be one VMKernel adaptor used for management traffic, one for vMotion
    # +2 for iSCSI
    if (!$this.iSCSI -and $(@($this.VMKernelAdaptors).length) -ne 2)
    {
      $this.vNowNetworkIssues += "There should be two VMKernelAdaptors configured `
        - one for host management traffic, one for vMotion traffic"
      $this.hostIssues += 1
    }
    elseif ($this.iSCSI -and $(@($this.VMKernelAdaptors).length) -ne 4)
    {
      $this.vNowNetworkIssues += "There should be four VMKernelAdaptors configured `
        - one for host management traffic, one for vMotion traffic, and two for iSCSI"
      $this.hostIssues += 1
    }
    else
    {
      $MgmtVMK = $this.VMKernelAdaptors | ? {($_.PortGroupName -eq "Management Network")}
      $vMotionVMK = $this.VMKernelAdaptors | ? {($_.PortGroupName -eq "vMotion")}
      $iSCSI1VMK = $this.VMKernelAdaptors | ? {($_.PortGroupName -eq "iSCSI 1")}
      $iSCSI2VMK = $this.VMKernelAdaptors | ? {($_.PortGroupName -eq "iSCSI 2")}

      if (!$MgmtVMK.ManagementTrafficEnabled -or $MgmtVMK.VMotionEnabled -or $MgmtVMK.FaultToleranceLoggingEnabled -or $MgmtVMK.VsanTrafficEnabled)
      {
        $this.vNowNetworkIssues += "Only host management traffic should be enabled on $($MgmtVMK.Name)"
        $this.hostIssues += 1
      }

      if ($vMotionVMK.ManagementTrafficEnabled -or !$vMotionVMK.VMotionEnabled -or $vMotionVMK.FaultToleranceLoggingEnabled -or $vMotionVMK.VsanTrafficEnabled)
      {
        $this.vNowNetworkIssues += "Only vMotion traffic should be enabled on $($vMotionVMK.Name)"
        $this.hostIssues += 1
      }

      if ($MgmtVMK.mtu -ne 1500)
      {
        $this.vNowNetworkIssues += "Management VMKernel should have MTU 1500, not $($MgmtVMK.mtu)"
        $this.hostIssues += 1
      }

      if ($vMotionVMK.mtu -ne 1500)
      {
        $this.vNowNetworkIssues += "vMotion VMKernel should have MTU 1500, not $($vMotionVMK.mtu)"
        $this.hostIssues += 1
      }

      if ($this.iSCSI)
      {
        if ($iSCSI1VMK.ManagementTrafficEnabled -or $iSCSI1VMK.VMotionEnabled -or $iSCSI1VMK.FaultToleranceLoggingEnabled -or $iSCSI1VMK.VsanTrafficEnabled)
        {
          $this.vNowNetworkIssues += "Only iSCSI traffic should be enabled on $($iSCSI1VMK.Name)"
          $this.hostIssues += 1
        }

        if ($iSCSI2VMK.ManagementTrafficEnabled -or $iSCSI2VMK.VMotionEnabled -or $iSCSI2VMK.FaultToleranceLoggingEnabled -or $iSCSI2VMK.VsanTrafficEnabled)
        {
          $this.vNowNetworkIssues += "Only iSCSI traffic should be enabled on $($iSCSI2VMK.Name)"
          $this.hostIssues += 1
        }

        if ($iSCSI1VMK.mtu -ne 9000)
        {
          $this.vNowNetworkIssues += "iSCSI 1 VMKernel should have MTU 9000, not $($iSCSI1VMK.mtu)"
          $this.hostIssues += 1
        }

        if ($iSCSI2VMK.mtu -ne 9000)
        {
          $this.vNowNetworkIssues += "iSCSI 2 VMKernel should have MTU 9000, not $($iSCSI2VMK.mtu)"
          $this.hostIssues += 1
        }
      }
    }

    # The SRIOV field contains information on which host NICs are configured for SR-IOV.
    if ($this.SRIOV -and ($this.SRIOV -ne "vmnic7,vmnic9"))
    {
      $this.vNowSRIOVIssue = $TRUE
      $this.hostIssues += 1
    }

    if ($this.AccountUnlockTime -ne 900)
    {
      $this.vNowSecurityIssues += "Account Unlock Time is $($this.AccountUnlockTime), not 900"
      $this.hostIssues += 1
    }
    if ($this.AccountLockFailures -ne 3)
    {
      $this.vNowSecurityIssues += "Account Lock Failures is $($this.AccountLockFailures), not 3"
      $this.hostIssues += 1
    }
    if ($this.ESXiShellIntTimeout -ne 900)
    {
      $this.vNowSecurityIssues += "ESXi Shell Interactive timeout is $($this.ESXiShellIntTimeout), not 900"
      $this.hostIssues += 1
    }
    if ($this.ESXiShellTimeout -ne 900)
    {
      $this.vNowSecurityIssues += "ESXi Shell timeout is $($this.ESXiShellTimeout), not 900"
      $this.hostIssues += 1
    }
    if ($this.BlockGuestBDPU -ne 1)
    {
      $this.vNowSecurityIssues += "Network Block Guest BPDU is $($this.BlockGuestBDPU), not 1"
      $this.hostIssues += 1
    }

    if ($this.SRIOV -and !$this.VMHostStartPolicy.Enabled)
    {
      $this.vNowVMHostStartPolicyIssue = $TRUE
      $this.hostIssues += 1
    }

    if ($this.TriggeredAlarms -or $this.ConfigIssues)
    {
      $this.vNowAlarmed = $TRUE
      $this.hostIssues += 1
    }
  }

  $this.hostIssues
}

# gatherAnalysisHTML returns a HTML string of all non compliant settings that
# have been detected on the host.
$HostClass | Add-Member -MemberType ScriptMethod -Name "gatherAnalysisHTML" -Value {

  $nonComplianceHost = ""
  $nonComplianceHost += "<H3>Host $($this.name)</H3>"

  if ($this.ConnectionState -eq "NotResponding")
  {
    $nonComplianceHost += "<div class=`"doubletab`">Host isn't currently `
      responding</div>"
    $nonComplianceHost += "<div class=`"doubletab`">WARNING - other host `
      information may be out of date.</div> <BR>"
  }

  if ($this.HTavailableIssue -eq $TRUE)
  {
    $nonComplianceHost += "<div class=`"doubletab`">HyperthreadAvailable is`
      set to $($this.HyperthreadAvailable)</div>"
    $nonComplianceHost += "<div class=`"doubletab`">HyperthreadAvailable`
      should be set to True</div>"
    $nonComplianceHost += "<div class=`"doubletab`">Please refer to the `
      `"Hyper-threading`" section of the VMware Deployment Design Guide `
      for more information</div> <BR>"
  }
  if ($this.HTactiveIssue -eq $TRUE)
  {
    $nonComplianceHost += "<div class=`"doubletab`">HyperthreadActive is set`
      to $($this.HyperthreadActive)</div>"
    $nonComplianceHost += "<div class=`"doubletab`">HyperthreadActive should`
      be set to True</div>"
    $nonComplianceHost += "<div class=`"doubletab`">Please refer to the `
      `"Hyper-threading`" section of the VMware Deployment Design Guide `
      for more information</div> <BR>"
  }
  if ($this.numCPUthreadsIssue -eq $TRUE)
  {
    $nonComplianceHost += "<div class=`"doubletab`">The total number of CPU`
      cores available (NumCpuThreads) is $($this.NumCpuThreads)</div>"
    $nonComplianceHost += "<div class=`"doubletab`">The number of CPU cores`
      allocated to VMs (ProvCpuCores) is $($this.ProvCpuCores)</div>"
    $nonComplianceHost += "<div class=`"doubletab`">NumCpuThreads should be at`
      least two more than ProvCpuCores</div>"
    $nonComplianceHost += "<div class=`"doubletab`">Please refer to the `
      `"Distribution of VMs across hosts`" section of the VMware Deployment `
      Design Guide for more information</div> <BR>"
  }

  $nonComplianceHost += "<div class=`"doubletab`">CPU(s):`
    $($this.hostCPUIssue)</div>"
  $nonComplianceHost += "<div class=`"doubletab`">To determine expected product performance, `
    please compare the <a href=`"http://www.cpubenchmark.net/singlethread.html`">benchmark `
    value</a> of your CPU(s) with the benchmark value of our reference `
    hardware (E5-2640 v2 @ 2.00GHz, or E5-2690 v3 @ 2.60GHz for Perimeta), `
    and refer to the `"Benchmarking performance`" section of the `
    VMware Deployment Design Guide.</div> <br>"

  if ($this.vNowESXiVersionIssue -eq $TRUE)
  {
    $nonComplianceHost += "<div class=`"doubletab`">Host is running ESXi `
      version $($this.ESXiVersion).</div>"
    $nonComplianceHost += "<div class=`"doubletab`">It should be version 6.0 or 6.5.</div> <BR>"
  }
  if ($this.vNowNumDatastoreIssue -eq $TRUE)
  {
    $nonComplianceHost += "<div class=`"doubletab`">Host should be configured with `
      at least two datastores.</div> <BR>"
  }
  if ($this.vNowNumDatastorePathIssues)
  {
    $nonComplianceHost += "<div class=`"doubletab`">Issue(s) with Storage configuration:</div>"
    foreach ($issue in $this.vNowNumDatastorePathIssues)
    {
      $nonComplianceHost += "<div class=`"doubletab`">$issue.</div>"
    }

    $nonComplianceHost += "<div class=`"doubletab`">Confirm the host is `
      `correctly cabled to its storage provider.</div><BR>"
  }
  if ($this.vNowMultiPathingIssue -eq $TRUE)
  {
    $nonComplianceHost += "<div class=`"doubletab`">Host should be configured with `
      Round Robin for each datastore.</div> <BR>"
  }
  if ($this.vNowDumpCollectorIssues)
  {
    $nonComplianceHost += "<div class=`"doubletab`">Issue(s) with DumpCollector configuration:</div>"
    foreach ($issue in $this.vNowDumpCollectorIssues)
    {
      $nonComplianceHost += "<div class=`"doubletab`">$issue</div>"
    }
    $nonComplianceHost += "<BR>"
  }
  if ($this.vNowSyslogIssues)
  {
    $nonComplianceHost += "<div class=`"doubletab`">Issue(s) with Syslog configuration:</div>"
    foreach ($issue in $this.vNowSyslogIssues)
    {
      $nonComplianceHost += "<div class=`"doubletab`">$issue</div>"
    }
    $nonComplianceHost += "<BR>"
  }
  if ($this.NTPIssues)
  {
    $nonComplianceHost += "<div class=`"doubletab`">Issue(s) with NTP configuration:</div>"
    foreach ($issue in $this.NTPIssues)
    {
      $nonComplianceHost += "<div class=`"doubletab`">$issue</div>"
    }
    $nonComplianceHost += "<BR>"
  }
  if ($this.vNowLockdownModeIssue)
  {
    $nonComplianceHost += "<div class=`"doubletab`">Host should be in normal `
      lockdown mode, not $($this.LockdownMode).</div> <BR>"
  }
  if ($this.vNowNetworkIssues)
  {
    $nonComplianceHost += "<div class=`"doubletab`">Issue(s) with Network configuration:</div>"
    foreach ($issue in $this.vNowNetworkIssues)
    {
      $nonComplianceHost += "<div class=`"doubletab`">$issue</div>"
    }
    $nonComplianceHost += "<BR>"
  }

  if ($this.vNowSRIOVIssue)
  {
    $nonComplianceHost += "<div class=`"doubletab`">Host has SR-IOV incorrectly configured.`
      It's using $($this.SRIOV) but should use vmnic7,vmnic9</div><BR>"
  }

  if ($this.vNowSecurityIssues)
  {
    $nonComplianceHost += "<div class=`"doubletab`">Issue(s) with Security configuration:</div>"
    foreach ($issue in $this.vNowSecurityIssues)
    {
      $nonComplianceHost += "<div class=`"doubletab`">$issue</div>"
    }
    $nonComplianceHost += "<BR>"
  }

  if ($this.vNowVMHostStartPolicyIssue -eq $TRUE)
  {
    $nonComplianceHost += "<div class=`"doubletab`">SR-IOV host should have automatic VM`
      startup configured but doesn't.</div><BR>"
  }

  if ($this.PhysicalAdapterIssues)
  {
    $nonComplianceHost += "<div class=`"doubletab`">"
    $nonComplianceHost += "There are the following issues with physical adapters:"
    $nonComplianceHost += "<ul>"

    foreach ($issue in $this.PhysicalAdapterIssues)
    {
      $nonComplianceHost += "<li>$issue</li>"
    }

    $nonComplianceHost += "</ul>"

    $nonComplianceHost += "You shold investigate the corresponding network cable connected to the host."

    $nonComplianceHost += "</div><BR>"

  }

  if ($this.vNowAlarmed)
  {
    $nonComplianceHost += "<div class=`"doubletab`">"

    $nonComplianceHost += "There are the following alarms on the host"
    $nonComplianceHost += "<ul>"

    foreach ($alarm in $this.TriggeredAlarms)
    {
      $alarmDef = Get-View -Id $alarm.Alarm

      $nonComplianceHost += "<li>$($alarmDef.Info.Name)</li>"
    }

    foreach ($cfgIssue in $this.ConfigIssues)
    {
      $nonComplianceHost += "<li>$($cfgIssue.FullFormattedMessage)</li>"
    }

    $nonComplianceHost += "</ul></div><BR>"
  }

  $nonComplianceHost
}

# gatherAnalysisTable returns an array of information of analysis data which
# lists the setting name, the setting value, and whether there's an issue.
$HostClass | Add-Member -MemberType ScriptMethod -Name "gatherAnalysisTable" -Value {
  $hostTablerow = @(@("Cluster", $this.Cluster, $FALSE),`
    @("Host", $this.Name, $FALSE),`
    @("HyperthreadAvailable", $this.HyperthreadAvailable, $this.HTavailableIssue),`
    @("HyperthreadActive", $this.HyperthreadActive, $this.HTactiveIssue),`
    @("NumCpuThreads", $this.NumCpuThreads, $this.numCPUthreadsIssue),`
    @("ProvCpuCores", $this.ProvCpuCores, $this.numCPUthreadsIssue), `
    @("has non-Perimeta guests", $this.nonPerimetaVMsPresent, $FALSE))

  $hostTablerow
}

# dump outputs information about a host to a txt file.
$HostClass | Add-Member -MemberType ScriptMethod -Name "dump" -Value {

  write-host "Dumping info for host $($this.name)"

  $filename = "$destDir/host_$($this.name)_details.txt"
  "SystemInfo:" | Out-File -filepath $filename -Append -Encoding ASCII
  $this.SystemInfo | Ft -autosize | out-string -width 4096 | Out-File -filepath $filename -Append -Encoding ASCII

  "Datastores:" | Out-File -filepath $filename -Append -Encoding ASCII
  $this.Datastores | select * | Ft -autosize | out-string -width 4096 | Out-File -filepath $filename -Append -Encoding ASCII

  "Network:" | Out-File -filepath $filename -Append -Encoding ASCII
  $this.Network | Ft -autosize | out-string -width 4096 | Out-File -filepath $filename -Append -Encoding ASCII

  "Network Drivers:" | Out-File -filepath $filename -Append -Encoding ASCII
  $this.NetworkDrivers | Ft -autosize | out-string -width 4096 | Out-File -filepath $filename -Append -Encoding ASCII

  "Distributed VM Networks:" | Out-File -filepath $filename -Append -Encoding ASCII
  $this.dVMnetworks | Select Name, VlanConfiguration | Ft -autosize | out-string -width 4096 | Out-File -filepath $filename -Append -Encoding ASCII

  "Non-distributed VM Networks:" | Out-File -filepath $filename -Append -Encoding ASCII
  $this.VMnetworks | Select Name, VirtualSwitch, VLanId | Ft -autosize | out-string -width 4096 | Out-File -filepath $filename -Append -Encoding ASCII

  "CPU Package info:" | Out-File -filepath $filename -Append -Encoding ASCII
  $this.CPUpackageinfo | Ft -autosize | out-string -width 4096 | Out-File -filepath $filename  -Append -Encoding ASCII

  "PCI Devices:" | Out-File -filepath $filename -Append -Encoding ASCII
  $this.PCI_devices | Ft -autosize | out-string -width 4096 | Out-File -filepath $filename -Append -Encoding ASCII
}

################################################################################
# Class VMClass
#
# VMClass defines a VM object that holds information about a VM. It contains
# methods to:
#   Get all relevant info about the VM (getInfo)
#   Analyse that info w.r.t. the VMware Deployment Design guide (analysis)
#   Collate that analysis so it can be used to generate an HTML analysis
#     document (gatherAnalysisHTML and gatherAnalysisTable)
#   Output all that information in a set of files (dump).
################################################################################
$VMClass = new-object psobject -Property @{
  # The following are fields for data we want to output.
  Host = $null
  Name = $null
  ProductName = $null
  ProductVendor = $null
  PowerState = $null
  NumCores = $null
  CoresPerSocket = $null
  ExpectedMHz = $null
  ResvMHz = $null
  Mem = $null
  ResvMem = $null
  LatencySensitivity = $null
  IsVCenter = $FALSE
  PerfCounters = $null
  HARestartPriority = $null
  NetworkAdapterType = $null
  DiskProvisioning = $null
  NUMAPreferHT = "False"
  NUMAAffinity = "NONE"
  CoalescingScheme = ""
  CPUAffinity = $null
  IOPSUnlimited = $null
  vAppEnabled = "disabled"
  SRIOV = $false
  isPerimeta = $false

  # The following fields hold analysis of above fields
  MHzIssue = $FALSE
  memIssue = $FALSE
  LSIssue = $FALSE
  NUMAprefIssue = $FALSE
  vAppIssue = $FALSE
  productVendorIssue = $FALSE
  NUMAAffinityIssue = $false
  CoalescingSchemeIssue = $false
  CoresPerSocketIssue = $false
  IOPSUnlimitedIssue = $false

  VMIssues = 0
}

# getInfo calls getInfoVMclass to populate the VMobject with information
$VMclass | Add-Member -MemberType ScriptMethod -Name "getInfo" -Value {
  param(
    [Parameter(Mandatory=$TRUE)]
    [object]$VM
  )
  $this.getInfoVMclass($VM)
}

# When getInfo is called on a new VMObject it takes a VM, interrogates it
# and populates the VMObject with info about the VM.
$VMclass | Add-Member -MemberType ScriptMethod -Name "getInfoVMclass" -Value {
  param(
    [Parameter(Mandatory=$TRUE)]
    [object]$VM
  )
  $this.name = $VM.name
  $this.Host = $VM.VMhost
  write-host "Collecting info for VM $($this.name)"

  if ($VM.ExtensionData.Config.VAppConfig.Product)
  {
    $this.ProductName = $VM.ExtensionData.Config.VAppConfig.Product[0].Name
    $this.productVendor = $VM.ExtensionData.Config.VAppConfig.Product[0].Vendor
  }

  $hostView = $this.Host | Get-View
  $hostMhz = $hostView.Hardware.CpuInfo.Hz / 1000000
  $hostMhz = [math]::truncate($hostMhz)
  $this.PowerState = $VM.PowerState
  $this.NumCores = $VM.NumCPU
  $this.CoresPerSocket = $VM.ExtensionData.Config.Hardware.NumCoresPerSocket
  $this.ResvMHz = $VM.VMResourceConfiguration.CpuReservationMhz
  $this.ExpectedMHz = $hostMhz * $VM.NumCPU / 2
  $this.ExpectedMHz = [math]::truncate($this.ExpectedMHz)
  $this.Mem = $VM.MemoryGB
  $this.ResvMem = $VM.VMResourceConfiguration.MemReservationGB
  $this.LatencySensitivity = $VM.ExtensionData.Config.LatencySensitivity.level
  $this.PerfCounters = $VM.ExtensionData.Config.VPMCEnabled

  if ($VM.ExtensionData.Config.Annotation -eq "VMware vCenter Server Appliance")
  {
    $this.IsVCenter = $TRUE
  }
  if ($VM.ExtensionData.Config.VAppConfig)
  {
    $this.vAppEnabled = "enabled"
  }

  $this.HARestartPriority = $VM.HARestartPriority
  if ($this.HARestartPriority -eq "ClusterRestartPriority")
  {
    $clusterDump = Get-Cluster -Server $viServer -Name "$($VM.VMHost.Parent)"
    $this.HARestartPriority = $clusterDump.HARestartPriority
  }

  $networkAdapters = $VM | Get-NetworkAdapter
  foreach ($networkAdapter in $networkAdapters)
  {
    $NetworkAdapterType = $networkAdapter.type
    $this.NetworkAdapterType += "$NetworkAdapterType "
    if ($networkAdapter.ExtensionData.SriovBacking)
    {
      $this.SRIOV = $true
    }
  }

  $vmview = $VM | Get-View
  foreach ($dev in $vmview.Config.Hardware.Device)
  {
    if (($dev.GetType()).Name -eq "VirtualDisk")
    {
      if ($dev.Backing.thinProvisioned -eq $TRUE)
      {
        $this.DiskProvisioning += "Thin "
      }
      else
      {
        $this.DiskProvisioning += "Thick "
      }
    }
  }

  $coalescingSchemes = @()
  foreach ($extraConfig in $VM.ExtensionData.Config.ExtraConfig)
  {
    if ($extraConfig.Key -eq "numa.vcpu.preferHT")
    {
      $this.NUMAPreferHT = $extraConfig.Value
    }
    if ($extraConfig.Key -eq "numa.nodeAffinity")
    {
      $this.NUMAAffinity = $extraConfig.Value
    }
    if ($extraConfig.Key -like "*.coalescingScheme")
    {
      $coalescingSchemes += "$($extraConfig.Key) = $($extraConfig.Value)"
    }
  }
  $this.CoalescingScheme = $coalescingSchemes -Join ", "

  if ($VM.ExtensionData.Config.CpuAffinity -eq $null)
  {
    $this.CPUAffinity = "Not Set"
  }
  else
  {
    $this.CPUAffinity = "Set"
  }

  $this.IOPSUnlimited = "True"
  foreach ($hd in (Get-HardDisk $VM))
  {
    if ($hd.ExtensionData.StorageIOAllocation.Limit -ne "-1")
    {
      $this.IOPSUnlimited = "False"
    }
  }
}

# analysis calls analysisVMclass to analyse the VMobject and then returns then
# returns the number of issues detected by analysisVMClass.
$VMclass | Add-Member -MemberType ScriptMethod -Name "analysis" -Value {
  $this.analysisVMClass()
  $this.VMIssues
}

# analysisVMClass analyses particular properties of the VM object to see if they
# are compliant with the VMware Deployment Design guide.
$VMclass | Add-Member -MemberType ScriptMethod -Name "analysisVMClass" -Value {

  # Allow some leeway with CPU reservation to allow for rounding errors
  # Stipulate that actual reservation should be within 0.1% of expected value
  if ([math]::abs($this.ResvMHz - $this.ExpectedMHz) -gt ($this.ExpectedMHz * 0.001))
  {
    $this.MHzIssue = $TRUE
    $this.VMIssues += 1
  }
  if ($this.ResvMem -ne $this.Mem)
  {
    $this.memIssue = $TRUE
    $this.VMIssues += 1
  }

  # Bypass checking of some settings for the vCenter VM, which we expect to be
  # hosted on the cluster for vNOW setups.
  if (!($vnow -and $this.IsVCenter))
  {
    if ($this.productVendor -ne $vmMswVendor)
    {
      $this.productVendorIssue = $TRUE
      $this.VMIssues += 1
    }
  }
}

# gatherAnalysisHTML returns the output of gatherAnalysisHTMLVMClass
$VMClass | Add-Member -MemberType ScriptMethod -Name "gatherAnalysisHTML" -Value {
  $this.gatherAnalysisHTMLVMClass()
}

# gatherAnalysisHTMLVMClass returns a HTML string of all non compliant settings
# on a VM.
$VMclass | Add-Member -MemberType ScriptMethod -Name "gatherAnalysisHTMLVMClass" -Value {

  $nonComplianceVM = $this.gatherAnalysisHTMLHeader()

  if ($this.MHzIssue -eq $TRUE)
  {
    $nonComplianceVM += "<div class=`"doubletab`">Expected CPU reservation`
      (ExpectedMHz) is $($this.ExpectedMHz) MHz</div>"
    $nonComplianceVM += "<div class=`"doubletab`">Actual CPU reservation`
      (ResvMHz) is $($this.ResvMHz) MHz</div>"
    $nonComplianceVM += "<div class=`"doubletab`">ResvMHz should equal`
      ExpectedMHz</div> <BR>"
  }
  if ($this.memIssue -eq $TRUE)
  {
    $nonComplianceVM += "<div class=`"doubletab`">Total memory (Mem) is`
      $($this.Mem) GB</div>"
    $nonComplianceVM += "<div class=`"doubletab`">Reserved memory (ResvMem)`
      is set to $($this.ResvMem) GB</div>"
    $nonComplianceVM += "<div class=`"doubletab`">ResvMem should equal`
      Mem</div> <BR>"
  }
  if ($this.vAppIssue -eq $TRUE)
  {
    $nonComplianceVM += "<div class=`"doubletab`">vApp options are disabled`
      </div>"
    $nonComplianceVM += "<div class=`"doubletab`">vApp options should be `
      enabled and populated</div>"
    $nonComplianceVM += "<div class=`"doubletab`">Please enable vApp options `
      on this VM via VM -> Actions -> Edit settings -> vApp options. Talk to a`
      Metaswitch CSE for repopulating the vApp options</div> <BR>"
  }
  if ($this.productVendorIssue -eq $TRUE)
  {
    $nonComplianceVM += "<div class=`"doubletab`">The product vendor is not `
      ""$vmMswVendor"" </div>"
    $nonComplianceVM += "<div class=`"doubletab`">Either this VM is not a `
      MetaSwitch product or the product vendor has been removed</div> <BR>"
  }
  if ($this.NUMAprefIssue -eq $True)
  {
    $nonComplianceVM += "<div class=`"doubletab`">NUMAPreferHT is set to`
      $($this.NUMAPreferHT)</div>"
    $nonComplianceVM += "<div class=`"doubletab`">NUMAPreferHT should be set `
      to True`</div> <br>"
  }
  $nonComplianceVM
}

# gatherAnalysisHTMLHeader returns a HTML header string for this VM.
$VMclass | Add-Member -MemberType ScriptMethod -Name "gatherAnalysisHTMLHeader" -Value {
  $nonComplianceVM = ""
  $nonComplianceVM += "<h3>VM $($this.name)"
  if ($this.ProductName)
  {
    $nonComplianceVM += " - $($this.ProductName)"
  }
  $nonComplianceVM += "</h3>"
  $nonComplianceVM
}

# gatherAnalysisTable returns the output of gatherAnalysisTableVMClass
$VMClass | Add-Member -MemberType ScriptMethod -Name "gatherAnalysisTable" -Value {
  $this.gatherAnalysisTableVMClass()
}

# gatherAnalysisTable returns an array of information of analysis data which
# lists the setting name, the setting value, and whether there's an issue.
$VMclass | Add-Member -MemberType ScriptMethod -Name "gatherAnalysisTableVMClass" -Value {
  $VMTablerow = @(@("Host", $this.Host.name, $FALSE), `
    @("VM name", $this.Name, $this.productVendorIssue),`
    @("Product", $this.ProductName, $false),`
    @("NumCores", $this.NumCores, $this.CoresPerSocketIssue),`
    @("CoresPerSocket", $this.CoresPerSocket, $this.CoresPerSocketIssue),`
    @("ExpectedMHz", $this.ExpectedMHz, $false), `
    @("ResvMHz", $this.ResvMHz, $this.MHzIssue),`
    @("Mem", $this.Mem, $false), `
    @("ResvMem", $this.ResvMem, $this.memIssue),`
    @("LatencySensitivity", $this.LatencySensitivity, $this.LSIssue),`
    @("NUMAPreferHT", $this.NUMAPreferHT, $this.NUMAprefIssue), `
    @("NUMAAffinity", $this.NUMAAffinity, $this.NUMAAffinityIssue), `
    @("IOPSUnlimited", $this.IOPSUnlimited, $this.IOPSUnlimitedIssue), `
    @("SRIOV", $this.SRIOV, $false), `
    @("vAppSettings", $this.vAppEnabled, $this.vAppIssue), `
    @("Is Perimeta", $this.isPerimeta, $null))

  $VMTableRow
}

# dump outputs a vmx file for each VM.
$VMclass | Add-Member -MemberType ScriptMethod -Name "dump" -Value {

  $VM = get-VM -name $this.name

  if ($this.productVendor -eq $vmMswVendor)
  {
    write-host "Dumping info for Metaswitch VM $VM..."

    # Strip out everything apart from alpha-numeric characters for the directory
    # to create.
    $cleanVmName = $VM -replace '[^\w]', ''
    mkdir $destDir/$cleanVmName > $null

    # Get the VM config file from the datastore, create a directory for
    # each VM with a vmx config file
    $vmViews = $VM | Get-View
    foreach ($vmView in $vmViews)
    {
      $vmxfile = $vmView.Config.Files.VmPathName
      $dsname = ($vmxfile -split '^.*\[(.*)\].*$')[1]
      $ds = $VM | Get-Datastore -Server $viServer -Name "$dsname" | Get-Unique
      $new_ds = New-PSDrive -Name ds -PSProvider VimDatastore -Root '/' -Location $ds
      Copy-DatastoreItem -Item "ds:$($vmxfile.split(']')[1].TrimStart(' '))" -Destination $destDir/$cleanVmName
      Remove-PSDrive $new_ds
    }
  }
  else
  {
    write-host "Not dumping info for non-Metaswitch VM $VM"
  }
}

################################################################################
# Class ABUClass
#
# ABUClass is a child of VMClass that holds additional information for an
# ABU product VM.
#
# It has methods that act as a common interface that runs each of the functional
# methods in VMclass as well as additonal methods in ABUClass.
################################################################################
$ABUClass = $VMclass.psobject.copy()
$ABUClass.isPerimeta = $FALSE

# getInfo calls getInfoVMclass to populate the VMobject with information
$ABUClass | Add-Member -MemberType ScriptMethod -force -Name "getInfo" -Value {
  param(
    [Parameter(Mandatory=$TRUE)]
    [object]$VM
  )
  $this.getInfoVMclass($VM)
}

# analysis returns the number of issues for the VM that were detected in
# analysisVMClass and analysisABUClass
$ABUClass | Add-Member -MemberType ScriptMethod -force -Name "analysis" -Value {
  $this.analysisVMClass()
  $this.analysisABUClass()
  $this.VMIssues
}

# analysisABUClass analyses config specific to ABU VMs to check if a VM
# is compliant with the VMware Deployment Design guide.
$ABUClass | Add-Member -MemberType ScriptMethod -Name "analysisABUClass" -Value {
  if ($this.LatencySensitivity -ne "normal")
  {
    $this.LSIssue = $True
    $this.VMIssues += 1
  }

  if ($this.vAppEnabled -ne "enabled")
  {
    $this.vAppIssue = $TRUE
    $this.VMIssues += 1
  }

  if ($this.NUMAPreferHT -ne $True)
  {
    $this.NUMAprefIssue = $True
    $this.VMIssues += 1
  }
}

# gatherAnalysisHTML adds the output of gatherAnalysisHTMLVMClass and
# gatherAnalysisHTMLABUClass and outputs the combined string.
$ABUClass | Add-Member -MemberType ScriptMethod -force -Name "gatherAnalysisHTML" -Value {
  $nonComplianceVMreport = $this.gatherAnalysisHTMLVMClass()
  $nonComplianceVMreport += $this.gatherAnalysisHTMLABUClass()
  $nonComplianceVMreport
}

# gatherAnalysisHTMLABUClass returns a string of all non compliant settings.
$ABUClass | Add-Member -MemberType ScriptMethod -Name "gatherAnalysisHTMLABUClass" -Value {
  $nonComplianceVMABU = ""
  if ($this.LSIssue -eq $True)
  {
    $nonComplianceVMABU += "<div class=`"doubletab`">LatencySensitivity is set to`
      $($this.LatencySensitivity)</div>"
    $nonComplianceVMABU += "<div class=`"doubletab`">LatencySensitivity should be`
      set to normal</div> <br>"
  }
  $nonComplianceVMABU
}

# gatherAnalysisTable returns the output of gatherAnalysisTableVMClass.
$ABUClass | Add-Member -MemberType ScriptMethod -force -Name "gatherAnalysisTable" -Value {
  $this.gatherAnalysisTableVMClass()
}

################################################################################
# Class SBUPerimetaClass
#
# SBUPerimetaClass is a child of VMClass that holds additional information for
# a Perimeta VM.
#
# It has methods that act as a common interface that runs each of the functional
# methods in VMclass as well as additonal methods in SBUPerimetaClass.
################################################################################
$SBUPerimetaClass = $VMclass.psobject.copy()
$SBUPerimetaClass.isPerimeta = $True

<#
.SYNOPSIS
SBUPerimetaCreate generates an SBUObject that is populated with information about VMs
that are Perimetas.
.DESCRIPTION
SBUPerimetaClass defines SBUobjects that hold information about a VM. It contains
methods to:
  Get all relevant info about the VM (getInfo)
  To analyse that info w.r.t. the VMware Deployment Design guide (analysis)
  To collate that analysis so it can be used to generate an HTML analysis
    document (gatherAnalysisHTML and gatherAnalysisTable)
  To output all that information in a set of files (dump).
SBUPerimetaClass is a child of VMClass and has additional parts to its methods that do
  additional analysis and creation of HTML strings.
.PARAMETER VM
The function takes a VMware VMobject object as a parameter.
#>
function SBUPerimetaCreate {
  param(
    [Parameter(Mandatory=$True)]
    [object]$VM
  )
  $SBUObject = $SBUPerimetaClass.psobject.copy()
  $SBUObject.getInfo($VM)
  $SBUObject
}

# getInfo calls getInfoVMclass to populate the VMobject with information
$SBUPerimetaClass | Add-Member -MemberType ScriptMethod -force -Name "getInfo" -Value {
  param(
    [Parameter(Mandatory=$True)]
    [object]$VM
  )
  $this.getInfoVMclass($VM)
}

# analysis returns the number of issues for the VM that were detected in
# analysisVMClass and analysisSBUPerimetaClass
$SBUPerimetaClass | Add-Member -MemberType ScriptMethod -force -Name "analysis" -Value {
  $this.analysisVMClass()
  $this.analysisSBUPerimetaClass()
  $this.VMIssues
}

# analysisSBUPerimetaClass analyses config specific to Perimeta VMs to check if a VM
# is compliant with the VMware Deployment Design guide.
$SBUPerimetaClass | Add-Member -MemberType ScriptMethod -Name "analysisSBUPerimetaClass" -Value {
  if ($this.LatencySensitivity -ne "normal")
  {
    $this.LSIssue = $True
    $this.VMIssues += 1
  }
  if ($this.SRIOV)
  {
    if ($this.NUMAAffinity -eq "NONE")
    {
      $this.NUMAAffinityIssue = $True
      $this.VMIssues += 1
    }
  }
  else
  {
    if ($this.NUMAAffinity -ne "NONE")
    {
      $this.NUMAAffinityIssue = $True
      $this.VMIssues += 1
    }
  }
  if ($this.CoalescingScheme -ne "")
  {
    $this.CoalescingSchemeIssue = $True
    $this.VMIssues += 1
  }
  if ($this.CoresPerSocket -ne $this.NumCores)
  {
    $this.CoresPerSocketIssue = $True
    $this.VMIssues += 1
  }
  if ($this.IOPSUnlimited -ne $True)
  {
    $this.IOPSUnlimitedIssue = $True
    $this.VMIssues += 1
  }

  if ($this.vAppEnabled -ne "enabled")
  {
    $this.vAppIssue = $TRUE
    $this.VMIssues += 1
  }

  if ($this.NUMAPreferHT -ne $True)
  {
    $this.NUMAprefIssue = $True
    $this.VMIssues += 1
  }
}

# gatherAnalysisHTML adds the output of gatherAnalysisHTMLVMClass and
# gatherAnalysisHTMLSBUPerimetaClass and outputs the combined string.
$SBUPerimetaClass | Add-Member -MemberType ScriptMethod -force -Name "gatherAnalysisHTML" -Value {
  $nonComplianceVM = $this.gatherAnalysisHTMLVMClass()
  $nonComplianceVM += $this.gatherAnalysisHTMLSBUPerimetaClass()
  $nonComplianceVM
}

# gatherAnalysisHTMLSBUPerimetaClass returns a string of all non compliant settings.
$SBUPerimetaClass | Add-Member -MemberType ScriptMethod -Name "gatherAnalysisHTMLSBUPerimetaClass" -Value {
  $nonComplianceVM = ""
  if ($this.LSIssue -eq $True)
  {
    $nonComplianceVM += "<div class=`"doubletab`">LatencySensitivity is set to `
      $($this.LatencySensitivity)</div>"
    $nonComplianceVM += "<div class=`"doubletab`">LatencySensitivity should be `
      set to normal</div> <br>"
  }
  if ($this.NUMAAffinityIssue -eq $True)
  {
    $nonComplianceVM += "<div class=`"doubletab`">NUMA Affinity is set to `
      $($this.NUMAAffinity)</div>"
    if ($this.SRIOV)
    {
      $nonComplianceVM += "<div class=`"doubletab`">NUMA Affinity should be set</div> <br>"
    }
    else
    {
      $nonComplianceVM += "<div class=`"doubletab`">NUMA Affinity should not be set</div> <br>"
    }
  }
  if ($this.CoalescingSchemeIssue -eq $True)
  {
    $nonComplianceVM += "<div class=`"doubletab`">Some ethernet coalescing schemes `
      are set: $($this.CoalescingScheme)</div>"
    $nonComplianceVM += "<div class=`"doubletab`">Ethernet coalescing schemes should `
      not be set</div> <br>"
  }
  if ($this.CoresPerSocketIssue -eq $True)
  {
    $nonComplianceVM += "<div class=`"doubletab`">Cores per socket is: `
      $($this.CoresPerSocket)</div>"
    $nonComplianceVM += "<div class=`"doubletab`">Number of cores is: `
      $($this.NumCores)</div>"
    $nonComplianceVM += "<div class=`"doubletab`">The number of cores per socket for `
      each virtual machine to be equivalent to the number of vCPUs used by the virtual `
    machine</div> <br>"
  }
  if ($this.IOPSUnlimitedIssue -eq $True)
  {
    $nonComplianceVM += "<div class=`"doubletab`">IOPS are limited on this virtual `
      machine.</div>"
    $nonComplianceVM += "<div class=`"doubletab`">IOPS should not be limited.</div> <br>"
  }
  $nonComplianceVM
}

# gatherAnalysisTable returns the output of gatherAnalysisTableVMClass.
$SBUPerimetaClass | Add-Member -MemberType ScriptMethod -force -Name "gatherAnalysisTable" -Value {
  $this.gatherAnalysisTableVMClass()
}

################################################################################
# Class SBUDCMClass
#
# SBUDCMClass is a child of VMClass that holds additional information for a
# DCM VM.
#
# It has methods that act as a common interface that runs each of the functional
# methods in VMclass as well as additonal methods in SBUDCMClass.
################################################################################
$SBUDCMClass = $VMclass.psobject.copy()
$SBUDCMClass.isPerimeta = $False

<#
.SYNOPSIS
SBUDCMCreate generates an SBUObject that is populated with information about a
DCM VM.
.DESCRIPTION
SBUDCMClass defines SBUobjects that hold information about a VM. It contains
methods to:
  Get all relevant info about the VM (getInfo)
  To analyse that info w.r.t. the VMware Deployment Design guide (analysis)
  To collate that analysis so it can be used to generate an HTML analysis
    document (gatherAnalysisHTML and gatherAnalysisTable)
  To output all that information in a set of files (dump).
SBUDCMClass is a child of VMClass and has additional parts to its methods that do
  additional analysis and creation of HTML strings.
.PARAMETER VM
The function takes a VMware VMobject object as a parameter.
#>
function SBUDCMCreate {
  param(
    [Parameter(Mandatory=$True)]
    [object]$VM
  )
  $SBUObject = $SBUDCMClass.psobject.copy()
  $SBUObject.getInfo($VM)
  $SBUObject
}

# getInfo calls getInfoVMclass to populate the VMobject with information
$SBUDCMClass | Add-Member -MemberType ScriptMethod -force -Name "getInfo" -Value {
  param(
    [Parameter(Mandatory=$True)]
    [object]$VM
  )
  $this.getInfoVMclass($VM)
}

# analysis returns the number of issues for the VM that were detected in
# analysisVMClass and analysisSBUDCMClass
$SBUDCMClass | Add-Member -MemberType ScriptMethod -force -Name "analysis" -Value {
  $this.VMIssues
}

# gatherAnalysisHTML just outputs the header - DCMs can't have any issues.
$SBUDCMClass | Add-Member -MemberType ScriptMethod -force -Name "gatherAnalysisHTML" -Value {
  $this.gatherAnalysisHTMLHeader()
}

# gatherAnalysisTable returns the output of gatherAnalysisTableVMClass.
$SBUDCMClass | Add-Member -MemberType ScriptMethod -force -Name "gatherAnalysisTable" -Value {
  $this.gatherAnalysisTableVMClass()
}

################################################################################
# Class Task
#
# Class to hold information about a task scheduled or performed on the VMware server.
# Certain tasks are not supported on Metaswitch VMs.
################################################################################
$Task = new-object psobject -Property @{

  cluster = $null
  host = $null
  action = $null
  last_run = $null
  next_run = $null
  vm = $null
  issue = $FALSE
  is_msw = $FALSE
}

# analysis analyses the task and checks whether it is an attempt to perform an
# unsupported action on the VM.
$Task | Add-Member -MemberType ScriptMethod -Name "analysis" -Value {

  $this.is_msw = IsMetaswitchVApp($this.vm)
  if ($this.is_msw)
  {
    # Suspending, cloning or snapshotting a Metaswitch VM is not supported.
    if (($this.action -eq "SuspendVM_Task") -or
        ($this.action -eq "CreateSnapshot_Task") -or
        ($this.action -eq "CloneVM_Task"))
    {
      $this.issue = $TRUE
    }
  }
}

# gatherAnalysisHTML gets HTML output for the analysis of the task
# and outputs the resulting string if there are problems.
$Task | Add-Member -MemberType ScriptMethod -force -Name "gatherAnalysisHTML" -Value {
  $nonComplianceTaskReport = ""

  if ($this.issue -eq $TRUE)
  {
    $nonComplianceTaskReport += "<H3>Task $($this.action) ($($this.host):$($this.vm.Name))</H3>"
    $nonComplianceTaskReport += "<div class=`"doubletab`">Task action $($this.action)`
      is not supported on a Metaswitch VM.</div>"
  }

  $nonComplianceTaskReport
}

# gatherAnalysisTable returns an array of information of analysis data which
# lists the task name, the VM it's set to act on, and whether there's an issue.
$Task | Add-Member -MemberType ScriptMethod -Name "gatherAnalysisTable" -Value {
  $TaskTableRow = @(@("Cluster", $this.cluster, $FALSE),`
    @("Host", $this.host, $FALSE),`
    @("VM", $this.vm.Name, $FALSE),`
    @("Last run", $this.last_run, $FALSE),`
    @("Next scheduled run", $this.next_run, $FALSE),`
    @("Action", $this.action, $this.issue),`
    @("Metaswitch VM", $this.is_msw, $this.issue))

  $TaskTableRow
}

# dump outputs information about a task to a txt file.
$Task | Add-Member -MemberType ScriptMethod -Name "dump" -Value {

  write-host "Dumping info for task $($this.action) on $($this.vm.Name)..."

  $filename = "$destDir/task_$($this.action)_$($this.vm.Name)_details.txt"
  "Cluster:" | Out-File -filepath $filename -Append -Encoding ASCII
  $this.cluster | select * | Ft -autosize | out-string -width 4096 | Out-File -filepath $filename -Append -Encoding ASCII

  "Host:" | Out-File -filepath $filename -Append -Encoding ASCII
  $this.host | select * | Ft -autosize | out-string -width 4096 | Out-File -filepath $filename -Append -Encoding ASCII

  "VM:" | Out-File -filepath $filename -Append -Encoding ASCII
  $this.vm | select * | Ft -autosize | out-string -width 4096 | Out-File -filepath $filename -Append -Encoding ASCII

  "Last run:" | Out-File -filepath $filename -Append -Encoding ASCII
  $this.last_run | out-string -width 4096 | Out-File -filepath $filename -Append -Encoding ASCII

  "Next run:" | Out-File -filepath $filename -Append -Encoding ASCII
  $this.next_run | out-string -width 4096 | Out-File -filepath $filename -Append -Encoding ASCII

  "Action:" | Out-File -filepath $filename -Append -Encoding ASCII
  $this.action | out-string -width 4096 | Out-File -filepath $filename -Append -Encoding ASCII
}

<#
.SYNOPSIS
CreateClusterObject generates a ClusterClass Object that is populated with
information about a cluster.
.DESCRIPTION
ClusterClass defines cluster objects that hold information about the cluster
and holds objects about all VMs and Hosts within it. It contains methods to:
  Get all relevant info about the cluster (getInfo).
  To analyse that info (analysis).
  To collate that analysis so it can be used to generate an HTML analysis
    document (gatherAnalysisHTML and gatherAnalysisTable).
  To output all that information in a set of files (dump).
Each cluster method also calls similar methods on VMs and Hosts within the
cluster.
.PARAMETER cluster
The function takes a VMware cluster object as a parameter
#>
function CreateClusterObject {
  param(
    [Parameter(Mandatory=$TRUE)]
    [object]$cluster
  )
  $newClusterObject = $ClusterClass.psobject.copy()
  $newClusterObject.getInfo($cluster)
  $newClusterObject
}

<#
.SYNOPSIS
CreateGhostClusterObject generates a GhostClusterClass object that is populated
with arrays of VMObjects and HostObjects.
.DESCRIPTION
GhostClusterClass defines cluster objects that holds objects about all VMs and
Hosts within it. It contains methods to:
  Call getinfo methods on hosts and VMs found in an object its passed (getInfo).
  Call analyse methods on the hosts and VMs in it's arrays (analysis).
  Collate that analysis so it can be used to generate an HTML analysis
    document (gatherAnalysisHTML and gatherAnalysisTable).
  Output all that information in a set of files (dump).
Each cluster method also calls similar methods on VMs and Hosts within the
cluster.
.PARAMETER unclusteredHosts
The function takes an object containing an array of hosts outside of clusters
as a parameter.
#>
function CreateGhostClusterObject {
  param(
    [Parameter(Mandatory=$TRUE)]
    [object]$unclusteredHosts
  )
  $newClusterObject = $GhostClusterClass.psobject.copy()
  $newClusterObject.getInfo($unclusteredHosts)
  $newClusterObject
}

<#
.SYNOPSIS
HostHACreate creates a HostHAObject and populates it with information.
.DESCRIPTION
ResourceHAClass defines ResourceHA objects that hold information about the
admission control setting. It contains methods to:
  Get all relevant info about the admission control setting (getInfo).
  Output that information as a string (dump).
.PARAMETER failoverHosts
It takes an array of objects containing info about the failover hosts as a
parameter.
#>
function HostHACreate {
  param(
    [Parameter(Mandatory=$TRUE)]
    [object]$failoverHosts
  )
  $HostHAObject = $HostHAClass.psobject.copy()
  $HostHAObject.getInfo($failoverHosts)
  $HostHAObject
}

<#
.SYNOPSIS
SlotHACreate creates a SlotHAObject and populates it with information.
.DESCRIPTION
ResourceHAClass defines ResourceHA objects that hold information about the
admission control setting. It contains methods to:
  Get all relevant info about the admission control setting (getInfo).
  Output that information as a string (dump).
.PARAMETER AdmisCtrlPolicy
It takes an object containing info about the admission control policy as a
parameter.
#>
function SlotHACreate {
  param(
    [Parameter(Mandatory=$TRUE)]
    [object]$AdmisCtrlPolicy
  )
  $SlotHAObject = $SlotHAClass.psobject.copy()
  $SlotHAObject.getInfo($AdmisCtrlPolicy)
  $SlotHAObject
}

<#
.SYNOPSIS
ResourceHACreate creates a ResourceHAObject and populates it with information.
.DESCRIPTION
ResourceHAClass defines ResourceHA objects that hold information about the
admission control setting. It contains methods to:
  Get all relevant info about the admission control setting (getInfo).
  Output that information as a string (dump).
.PARAMETER AdmisCtrlPolicy
It takes an object containing info about the admission control policy as a
parameter.
#>
function ResourceHACreate {
  param(
    [Parameter(Mandatory=$TRUE)]
    [object]$AdmisCtrlPolicy
  )
  $ResourceHAObject = $ResourceHAClass.psobject.copy()
  $ResourceHAObject.getInfo($AdmisCtrlPolicy)
  $ResourceHAObject
}

<#
.SYNOPSIS
InvalidHACreate creates a InvalidHAObject.
.DESCRIPTION
ResourceHAClass defines ResourceHA objects that hold information about the
admission control setting. It contains methods to:
  Output that information as a string (dump).
#>
function InvalidHACreate {
  $InvalidHAObject = $ResourceHAClass.psobject.copy()
  $InvalidHAObject
}

<#
.SYNOPSIS
CreateHostObject creates a HostObject that is populated with info about a
host.
.DESCRIPTION
HostClass defines host objects that hold information about the host. It
contains methods to:
  Get all relevant info about the host (getInfo).
  Analyse that info (analysis).
  Collate that analysis so it can be used to generate an HTML analysis
    document (gatherAnalysisHTML and gatherAnalysisTable).
  Output all that information in a set of files (dump).
.PARAMETER VMHost
The function takes a VMware VMHost object as a parameter.
#>
function CreateHostObject {
  param(
    [Parameter(Mandatory=$TRUE)]
    [object]$VMHost
  )
  $newHostObject = $hostClass.psobject.copy()
  $newHostObject.getInfo($VMHost)
  $newHostObject
}

<#
.SYNOPSIS
CreateVMObject generates a VMObject that is populated with information about a
VM.
.DESCRIPTION
VMClass defines VM objects that hold information about the VM. It contains
methods to:
  Get all relevant info about the VM (getInfo)
  Analyse that info w.r.t. the VMware Deployment Design guide (analysis)
  Collate that analysis so it can be used to generate an HTML analysis
    document (gatherAnalysisHTML and gatherAnalysisTable)
  Output all that information in a set of files (dump).
.PARAMETER VM
The function takes a VMware VMobject object as a parameter.
#>
function CreateVMObject {
  param(
    [Parameter(Mandatory=$TRUE)]
    [object]$VM
  )
  $newVMObject = $VMclass.psobject.copy()
  $newVMObject.getInfo($VM)
  $newVMObject
}

<#
.SYNOPSIS
CreateABUObject generates an ABUObject that is populated with information about
VMs that run ABU products.
.DESCRIPTION
ABUClass defines ABUobjects that hold information about a VM. It contains
methods to:
  Get all relevant info about the VM (getInfo)
  Analyse that info w.r.t. the VMware Deployment Design guide (analysis)
  Collate that analysis so it can be used to generate an HTML analysis
    document (gatherAnalysisHTML and gatherAnalysisTable)
  Output all that information in a set of files (dump).
ABUClass is a child of VMClass and has additional parts to its methods that do
  additional analysis and creation of HTML strings.
.PARAMETER VM
The function takes a VMware VMobject object as a parameter.
#>
function CreateABUObject {
  param(
    [Parameter(Mandatory=$TRUE)]
    [object]$VM
  )
  $ABUObject = $ABUClass.psobject.copy()
  $ABUObject.getInfo($VM)
  $ABUObject
}

<#
.SYNOPSIS
createVMObjects takes a selection of VMs and re-gets them all by ID.  It then
creates and returns an array of VM objects.
.PARAMETER VMs
The function takes an array of VMware VMobject objects as a parameter.
#>
function createVMObjects {
  param(
    [Parameter(Mandatory=$TRUE)]
    [object]$VMs
  )
  $output = @()

  foreach ($VM in $VMs)
  {
    # VM is got for a second time. This is due to some ways of getting VMobjects
    # leaving out HA setting info on the VM, the VM has to be got by id in
    # order to retain HA setting info.
    $VM = get-VM -id "$($VM.id)"

    # If more than one VM has the same name and is returned then just take the
    # first one.
    if ($VM.GetType().IsArray)
    {
      $VM = $VM[0]
    }

    if ($(IsABUVM($VM)) -eq $TRUE)
    {
      $VMObject = CreateABUObject($VM)
    }
    elseif ($(IsSBUPerimetaVM($VM)) -eq $true)
    {
      $VMObject = SBUPerimetaCreate($VM)
    }
    elseif ($(IsSBUDCMVM($VM)) -eq $true)
    {
      $VMObject = SBUDCMCreate($VM)
    }
    else
    {
      $VMObject = CreateVMObject($VM)
    }
    $output += $VMObject
  }
  $output
}

<#
.SYNOPSIS
CreateTask creates a blank Task object.
#>
function CreateTask {
  $newTask = $Task.psobject.copy()
  $newTask
}

<#
.SYNOPSIS
IsABUVM tests whether a VM is an instance of an ABU product.  It returns true
if so, otherwise false.
.PARAMETER VM
The function takes a VMware VMobject object as a parameter.
#>
function IsABUVM {
  param(
    [Parameter(Mandatory=$TRUE)]
    [object]$VM
  )
  $ABUproducts = @($vmName_ABU_RPAS,
                   $vmName_ABU_ESAP,
                   $vmName_ABU_AMS,
                   $vmName_ABU_OBS,
                   $vmName_ABU_SAS,
                   $vmName_ABU_MVS,
                   $vmName_ABU_MVOS,
                   $vmName_ABU_MVD,
                   $vmName_ABU_CFS,
                   $vmName_ABU_EAS,
                   $vmName_ABU_MRS)

  $output = $FALSE

  if ($VM.ExtensionData.Config.VAppConfig.Product -and
      $ABUproducts.Contains($VM.ExtensionData.Config.VAppConfig.Product[0].Name))
  {
    $output = $TRUE
  }

  $output
}

<#
.SYNOPSIS
IsSBUPerimetaVM tests whether a VM is a Perimeta.  It returns true if so,
otherwise false.
.PARAMETER VM
The function takes a VMware VMobject object as a parameter.
#>
function IsSBUPerimetaVM {
  param(
    [Parameter(Mandatory=$True)]
    [object]$VM
  )
  $SBUPerimetaProducts = @($vmName_SBU_Perimeta)

  $output = $FALSE

  if ($VM.ExtensionData.Config.VAppConfig.Product -and
      $SBUPerimetaProducts.Contains($VM.ExtensionData.Config.VAppConfig.Product[0].Name))
  {
    $output = $TRUE
  }

  $output
}

<#
.SYNOPSIS
IsSBUDCMVM tests whether a VM is a DCM.  It returns true if so, otherwise
false.
.PARAMETER VM
The function takes a VMware VMobject object as a parameter.
#>
function IsSBUDCMVM {
  param(
    [Parameter(Mandatory=$True)]
    [object]$VM
  )
  $SBUDCMProducts = @($vmName_SBU_DCM)

  $output = $FALSE

  if ($VM.ExtensionData.Config.VAppConfig.Product -and
      $SBUDCMProducts.Contains($VM.ExtensionData.Config.VAppConfig.Product[0].Name))
  {
    $output = $TRUE
  }

  $output
}

<#
.SYNOPSIS
IsMetaswitchVApp tests whether a VM is a Metaswitch Virtual Appliance.  It
returns true if so, otherwise false.
.PARAMETER VM
The function takes a VMware VMobject object as a parameter.
#>
function IsMetaswitchVApp {
  param(
    [Parameter(Mandatory=$True)]
    [object]$VM
  )

  $output = $FALSE

  if ($VM.ExtensionData.Config.VAppConfig.Product -and
      $VM.ExtensionData.Config.VAppConfig.Product[0].Name -Match $vmNameCommonString)
  {
    $output = $TRUE
  }

  $output
}

<#
.SYNOPSIS
IsPairedProduct tests whether a VM is an instance of a product that runs as
an active/standby pair.  It returns true if so, otherwise false.
.PARAMETER VM
The function takes a VMware VMobject object as a parameter.
#>
function IsPairedProduct {
  param(
    [Parameter(Mandatory=$TRUE)]
    [object]$VM
  )
  $pairedProducts = @($vmName_ABU_CFS,
                      $vmName_ABU_MVD,
                      $vmName_SBU_Perimeta)

  $output = $FALSE

  if ($VM.ExtensionData.Config.VAppConfig.Product -and
      $pairedProducts.Contains($VM.ExtensionData.Config.VAppConfig.Product[0].Name))
  {
    $output = $TRUE
  }

  $output
}

<#
.SYNOPSIS
validateDRSrules takes the DRS rules of a cluster and returns only the
Anti-affinity rules that are enabled and involve two systems.
.PARAMETER DRSrules
The function takes the DRSrules object of a VMware cluster as a parameter.
#>

function validateDRSrules {
  param(
    [Parameter(Mandatory=$TRUE)]
    [array]$DRSrules
  )
  foreach ($DRS_rule in $DRSrules)
  {
    if ($DRS_rule.type -eq "VMAntiAffinity")
    {
      if ($DRS_rule.enabled -eq $TRUE)
      {
        if ($DRS_rule.VMIds.length -eq 2)
        {
          $validDRSrules += ,$DRS_rule
        }
      }
    }
  }
  $validDRSrules
}

<#
.SYNOPSIS
returnNumIssues takes an array of clusters, totals the number of configuration
issues in each cluster, and returns that number.
.PARAMETER clustersArray
The function takes the global array of ClusterClass objects as a parameter.
#>
function returnNumIssues {
  param(
    [Parameter(Mandatory=$TRUE)]
    [array]$clustersArray
  )
  $totalIssues = 0
  foreach ($cluster in $clustersArray)
  {
    $totalIssues += $cluster.clusterIssues
  }
  $totalIssues
}

<#
.SYNOPSIS
returnNumTaskIssues takes an array of tasks, totals the number of configuration
issues, and returns that number.
.PARAMETER tasksArray
The function takes the global array of Task objects as a parameter.
#>
function returnNumTaskIssues {
  param(
        [Parameter(Mandatory=$TRUE)]
        [array]$tasksArray
  )
  $totalIssues = 0
  foreach ($task in $tasksArray)
  {
    if ($task.issue)
    {
      $totalIssues += 1
    }
  }
  $totalIssues
}

<#
.SYNOPSIS
VMDetails takes a list of VMs and, for each VM, creates a VMDetail object
populated with relevant info. Each VMDetail object is added to the VMDetails
array which is then returned.
.PARAMETER globalVMlist
The function takes the global array of VMClass objects as a parameter.
#>
function VMDetails {
  param(
    [Parameter(Mandatory=$TRUE)]
    [object]$globalVMList
  )
  $VMDetails = @()
  foreach ($VMObject in $globalVMList)
  {
    $VMDetail = $VMObject | Select Host, Name, ProductName, PowerState,`
      NumCores, CoresPerSocket, ExpectedMHz, ResvMHz, Mem, ResvMem,`
      LatencySensitivity, NUMAPreferHT, NUMAAffinity, CoalescingScheme,`
      IOPSUnlimited, PerfCounters, CPUAffinity, DiskProvisioning,`
      NetworkAdapterType, HARestartPriority

    $VMDetails += ,$VMDetail
  }
  $VMDetails
}

<#
.SYNOPSIS
HostDetails takes a list of hosts and, for each host, creates a HostDetail
object populated with relevant info. Each HostDetail object is added to the
HostDetails array which is then returned.
.PARAMETER globalHostlist
The function takes the global array of HostClass objects as a parameter.
#>
function HostDetails {
  param(
    [Parameter(Mandatory=$TRUE)]
    [object]$globalHostList
  )

  $HostDetails = @()
  foreach ($HostObject in $globalHostList)
  {
    $HostDetail = $HostObject | Select Cluster, Name, NumCpuSockets,`
      NumCpuCores, NumCpuThreads, ProvCpuCores, RAM, ProvRAM,`
      HyperthreadAvailable, HyperthreadActive, ESXiVersion

    $HostDetails += ,$HostDetail
  }
  $HostDetails
}

<#
.SYNOPSIS
createAnalysisArrays takes an array of objects, and creates an array that holds
analysis information for each object in a format that can be easily read by the
function createTableHTMLcode. Each array is added to analysisArrays which is
then returned.
.PARAMETER objectsArray
The function takes an array of objects from the same class (either ClusterClass,
HostClass or VMClass objects) as a parameter.
#>
function createAnalysisArrays {
  param(
    [Parameter(Mandatory=$TRUE)]
    [array]$objectsArray
  )

  $analysisArrays = @()
  foreach ($object in $objectsArray)
  {
    $analysisArray = $object.gatherAnalysisTable()
    $analysisArrays += ,$analysisArray
  }

  # Ensure return array is not unrolled if it only contains a single array
  ,$analysisArrays
}

<#
.SYNOPSIS
createHTMLtableHeader takes an analysisArray and returns a string containing
an HTML table header row with a heading for each field in the analysisArray.
.PARAMETER analysisArray
The function takes an array of arrays as a parameter.
#>
function createHTMLtableHeader {
  param(
        [Parameter(Mandatory=$TRUE,Position=1)]
        [array]$analysisArray
  )
  $HTMLheaderString = "<tr>"
  foreach ($analysisField in $analysisArray)
  {
    $HTMLheaderString += "<th>$($analysisField[0])</th>"
  }
  $HTMLheaderString += "</tr>"
  $HTMLheaderString
}

<#
.SYNOPSIS
# createHTMLtableRow takes an analysisArray of a cluster, host or VM and returns
# an HTML string of a table row containing that analysis, formatting the table
cells depending on whether that setting is compliant or not.
.PARAMETER analysisArrays
The function takes an array of arrays of arrays as a parameter.
#>
function createHTMLtableRow {
  param(
        [Parameter(Mandatory=$TRUE,Position=1)]
        [array]$analysisArray
  )
  $HTMLrowString = "<tr>"
  foreach ($analysisField in $analysisArray)
  {
    if ($analysisField[2] -eq $TRUE)
    {
      $HTMLrowString += "<td style=`"background-color: red`">"
    }
    else
    {
      $HTMLrowString += "<td>"
    }
    $HTMLrowString += "$($analysisField[1])</td>"
  }
  $HTMLrowString += "</tr>"
  $HTMLrowString
}

<#
.SYNOPSIS
createTableHTMLcode takes a array of "analysisArray"s and creates a string of
HTML for a table, starting with creating a header, then creating a row for each
analysisArray, and then returning the table HTML string.
.PARAMETER analysisArrays
The function takes an array of arrays of arrays as a parameter.
#>
function createTableHTMLcode {
  param(
        [Parameter(Mandatory=$TRUE,Position=1)]
        [array]$analysisArrays
  )
  $row_count = 0
  $HTMLtableString = "<table>"
  foreach ($analysisArray in $analysisArrays)
  {
    # Print the headers at the top and after every 20 rows
    if (($row_count % 20) -eq 0)
    {
      $HTMLtableString += createHTMLtableHeader($analysisArrays[0])
    }

    $HTMLtableString += createHTMLtableRow($analysisArray)
    $row_count++
  }
  $HTMLtableString += "</table>"
  $HTMLtableString
}

<#
.SYNOPSIS
createAnalysisReport generates the complete HTML report and outputs it.
#>
function createAnalysisReport {

  # Total number of configuration non-compliance issues are reported.
  $numberIssues = returnNumIssues($clustersArray)
  if ($globalBadTaskList)
  {
    $numberTaskIssues = returnNumTaskIssues($globalBadTaskList)
  }
  else
  {
    $numberTaskIssues = 0
  }

  $totalIssues = $numberIssues + $numberTaskIssues
  # analysis is arranged into arrays that can be fed into createTableHTMLcode.
  $analysisClusterArrays = createAnalysisArrays($clustersArray)
  $analysisHostArrays = createAnalysisArrays($globalHostList)
  $analysisVMArrays = createAnalysisArrays($globalVMList)

  # analysis is turned into a table written in html code.
  $clusterTableHTML = createTableHTMLcode($analysisClusterArrays)
  $hostTableHTML = createTableHTMLcode($analysisHostArrays)
  $VMTableHTML = createTableHTMLcode($analysisVMArrays)

  # the HTML code is turned into XML code.
  $clusterTableXML = [xml]$clusterTableHTML
  $hostTableXML = [xml]$hostTableHTML
  $VMTableXML = [xml]$VMTableHTML

  # Do all the above for tasks with issues.
  if ($globalBadTaskList)
  {
    $analysisTaskArrays = createAnalysisArrays($globalBadTaskList)
    $TaskTableHTML = createTableHTMLcode($analysisTaskArrays)
    $TaskTableXML = [xml]("<div><h2>Tasks</h2>" + $TaskTableHTML + "</div>")
  }
  else
  {
    $TaskTableXML = [xml]"<table />"
  }

  $compliance = "is not"
  if ($totalIssues -eq 0)
  {
    $compliance = "is"
  }

  $nonCompliantSettings = @()
  foreach ($cluster in $clustersArray)
  {
    if ($cluster.clusterIssues)
    {
      $nonCompliantSettings += $cluster.gatherAnalysisHTML()
    }

    if ($vnow -and $($cluster.clusterVNowIssues))
    {
      write-host "$(@($cluster.clusterVNowIssues).length) issues found"

      foreach ($vNowIssue in $cluster.clusterVNowIssues)
      {
        write-host $vNowIssue
      }
    }
  }

  if ($numberTaskIssues -gt 0)
  {
    $nonCompliantSettings += @"
      <H2>Tasks</H2>
      <div class=`"tab`">The following tasks are not supported on the systems
      they affect.</div>
"@
  }

  foreach ($task in $globalBadTaskList)
  {
    $nonCompliantSettings += $task.gatherAnalysisHTML()
  }

  $body = @"
  <H1 style="text-decoration: underline">VMWare analysis configuration report</H1>
  <H2>$vCenter has $totalIssues instances of non-compliance</H2>
  <H2>$vCenter $compliance compliant with the VMware Deployment Design Guide</H2>
"@

  if ($script:NoVSphereHA)
  {
    $body += @"
  <H2>WARNING: $vCenter doesn't have a license for vSphere HA.
  Please check the Metaswitch Products VMWare Deployment Design Guide for
  limitations when deploying Metaswitch products without vSphere HA.</H2>
"@
  }

  $body += @"
  $($clusterTableXML.innerxml)
  $($hostTableXML.innerxml)
  $($VMTableXML.innerxml)
  $($TaskTableXML.innerxml)
  $nonCompliantSettings
"@

  ConvertTo-HTML -head $head -Body "$body" | Out-File $destDir\analysis_report.html
  $totalIssues
}

# $head is css formatting for the html analysis report.
$head = @'
<Title>VMware Analysis Report</Title>
<style>
body
{
  background-color:#FFFFFF;
  font-family:Tahoma;
  font-size:12pt;
}
H1, H2
{
  margin-left:40px
}
H3
{
  margin-left:80px
}
td, th
{
  border:1px solid black;
  border-collapse:collapse
}
th
{
  color:white;
  background-color:black;
}
table, tr, td, th { padding: 5px; margin-left:40px }

.tab { margin-left: 80px; }
.doubletab { margin-left: 120px; }

</style>
'@

<#
.SYNOPSIS
findUnclusteredHosts detects any hosts outside of clusters in the vCenter. If
there are any such hosts these are
#>
function findUnclusteredHosts {
  $clusteredVMhosts = @()
  $unclusteredVMhosts = @()

  $VMHosts = get-VMhost -Server $viServer

  foreach ($cluster in $clusters)
  {
    $clusteredVMhosts += $cluster | get-VMhost
  }

  foreach ($VMhost in $VMhosts)
  {
    $hostIsUnclustered = $TRUE
    # Each host is compared to all hosts within all clusters. If it is not
    # present then it is added to a list of unclustered hosts.
    foreach ($clusteredVMhost in $clusteredVMhosts)
    {
      if ($clusteredVMhost -eq $VMhost)
      {
        $hostIsUnclustered = $FALSE
        break
      }
    }
    if ($hostIsUnclustered)
    {
      $unclusteredVMhosts += $VMhost
    }
  }

  # If there are unclustered hosts a new "ghost" cluster object is created to
  # hold this list of hosts.
  if ($unclusteredVMhosts)
  {
    $unclusteredHosts = new-object psobject
    $unclusteredHosts | Add-Member -type NoteProperty -name type -value "unclusteredHosts"
    $unclusteredHosts | Add-Member -type NoteProperty -name hosts -value $unclusteredVMhosts

    $unclusteredHosts
  }
}

# The script starts here.
if (!$vCenter)
{
  "vCenter server address has not been provided."
  exit 1
}

$vCenterIP = ""

if (!$destDir)
{
  "A diags directory has not been provided."
  exit 1
}

if (Test-Path -Path $destDir)
{
  "Folder already exists, you must provide a directory which does not exist."
  exit 1
}

# We get a few deprecation warnings when we do "select *". These are fine - when
# they get removed the selects will still work. but the warnings don't look good.
# So stop them showing here.
Set-PowerCLIConfiguration -Scope Session -confirm:$false -DisplayDeprecationWarnings $false | Out-Null

# Clear any previously connected servers. We have seen cases of duplicate VM Ids
# over separate vCenters.
if ($global:DefaultVIServers.Count -gt 0)
{
  Disconnect-VIServer -Server * -confirm:$false
}

$viServer = Connect-VIServer -Server $vCenter

# vNOW checking can't be done if cluster(s)/VM(s) are specified, as we need to
# check the entire vCenter
if ($vnow -and ($clusterName -or $vmName))
{
  write-host "Not permitted to specify clusters or VMs with vNOW checking"
  exit 1
}

# Does this rig have vSphere HA licensed? Checking is slightly different if so.
# All vSphere editions include vSphere HA apart from the vSphere Essentials
# Kit. To test for Essentials, take the first host's license key and see what
# kind of license that is. This test assumes that all hosts have the same kind
# of license but that's a reasonable assumption in this case because the
# vSphere Essentials kit is sold exclusively as a pack of three host licenses
# and a special locked-down vCenter Server license and cannot be combined with
# other licenses. (In theory Essentials could be mixed with Essentials Plus,
# but Essentials Plus is similarly sold exclusively as a three host plus
# vCenter Server kit so there's no reason to mix and match.) Note that we have
# to check the host licenses rather than the vCenter Server license because the
# vCenter Server Essentials license is common between the vSphere Essentials
# Kit (which doesn't support vSphere HA) and the vSphere Essentials Plus Kit
# (which does).
$script:NoVSphereHA = $FALSE
$hostLicenseKey = get-vmhost | Select-Object -first 1 | Select -expandProperty LicenseKey

if ($hostLicenseKey)
{
  $serviceInstance = Get-View ServiceInstance
  $licenseManager = Get-View $serviceInstance.content.LicenseManager

  if ($licenseManager.licenses | ? {(($_.LicenseKey -eq $hostLicenseKey) -and ($_.Name -match "Essentials$"))})
  {
    $script:NoVSphereHA = $TRUE
  }
}

# First look for the cluster (if one has been provided).
if (!$clusterName)
{
  # Get an array of the clusters present.  If there is only one cluster,
  # this syntax (initialising as an empty array and then adding to it) ensures
  # that we still end up with an array.  We need an array in order to add our
  # 'ghost' cluster containing unclustered hosts.
  $clusters = @()
  $clusters += Get-Cluster -Server $viServer

  # If there are hosts outside of clusters these are put in a unclusteredHosts
  # object and added to the clusters array.
  $unclusteredHosts = findUnclusteredHosts
  if ($unclusteredHosts)
  {
    $clusters += $unclusteredHosts
  }
}
else
{
  $clusters = Get-Cluster -Server $viServer -Name "$clusterName"
  if (!$clusters)
  {
    write-host "No clusters with that name found"
    exit 1
  }
}

# If no clusters or hosts have been found, the script will exit.
if (!$clusters)
{
  write-host "No clusters or hosts found"
  exit 1
}
elseif ($vnow)
{
  # vNOW should have all hosts in one cluster, or split into two clusters - one
  # for non-SRIOV and a second for SRIOV
  if ($unclusteredHosts)
  {
    write-host "Can't perform vNOW checking with hosts not in a cluster"
    exit 1
  }
  elseif ($clusters.length -gt 2)
  {
    write-host "Can't perform vNOW checking with more than two clusters"
    exit 1
  }
}

mkdir $destDir > $null

$clustersArray = @()
$globalHostList = @()
$globalVMList = @()
$globalBadTaskList = @()

# This creates a clusterObject and all the VM and Host objects within it.
foreach ($cluster in $clusters)
{
  # Clusters are checked to see if they are a "ghost" cluster the script has
  # generated to hold all hosts outside of clusters.
  if ($cluster.type -ne "unclusteredHosts")
  {
    # Cluster objects are created and populated with information. VM and Host
    # Objects on the cluster are also created and populated with information.
    $clusterObject = CreateClusterObject($cluster)
  }
  else
  {
    # If the cluster has unclustered host objects in it a ghost cluster object
    # is created that holds information on the hosts and VMs outside of
    # clusters.
    $clusterObject = CreateGhostClusterObject($cluster)
  }
  $clustersArray += $clusterObject

  # Configuration settings on clusters, hosts and vms are analysed.
  $clusterObject.analysis()

  # detailed txt files for cluster and host and vmx files for VM are generated.
  $clusterObject.dump()
}

# Any VM that represents a Metaswitch virtual appliance should be
# checked to verify that recent tasks performed on them are valid.
write-host "Checking tasks..."
$ParameterSet = @{
  Id = "ServiceInstance"
}
$ServiceInstance = Get-View @ParameterSet
$TaskManager = Get-View $ServiceInstance.Content.TaskManager
Get-Vm | ForEach-Object {
  if ($(IsMetaswitchVApp($_)) -eq $TRUE)
  {
    $vm = $_
    $mor = New-Object Vmware.Vim.ManagedObjectReference
    $mor.type = "VirtualMachine"
    $mor.value = $vm.Id
    $EntityFilter = New-Object VMware.Vim.TaskFilterSpecByEntity
    $EntityFilter.entity = $mor
    $TaskFilter = New-Object VMware.Vim.TaskFilterSpec
    $TaskFilter.entity = $EntityFilter
    $TaskCollector = Get-View ($TaskManager.CreateCollectorForTasks($TaskFilter))
    $tasks = $TaskCollector.ReadNextTasks.Invoke(200)
    $tasks | ForEach-Object {
      if ($_)
      {
        $taskobj = CreateTask
        $taskobj.host = $vm.Host
        $taskobj.cluster = $vm.Host.Parent
        $taskobj.action = $_.Name
        $taskobj.last_run = $_.startTime
        $taskobj.next_run = $_.nextRunTime
        $taskobj.vm = $vm
        $taskobj.analysis()
        if ($taskobj.issue -eq $TRUE)
        {
          $globalBadTaskList += $taskobj
          $taskobj.dump()
        }
      }
    }
    $TaskCollector.DestroyCollector()
  }
}

# Analyse scheduled tasks to check that they are allowed.
$ScheduledTaskManager = $ServiceInstance |
ForEach-Object {
  if ($_)
  {
    Get-View -Id $_.Content.ScheduledTaskManager
  }
}

if ($ScheduledTaskManager | Select -ExpandProperty ScheduledTask)
{
  $ExpandedTask = Get-View -Id ($ScheduledTaskManager | Select -ExpandProperty ScheduledTask)
  $ExpandedTask | ForEach-Object {
    if ($_)
    {
      # Only analyse tasks with an associated VM.
      if ($_.Info.TaskObject.type -eq "VirtualMachine")
      {
        $taskobj = CreateTask
        $taskobj.action = $_.Info.Action.Name
        $taskobj.last_run = $_.Info.startTime
        $taskobj.next_run = $_.Info.nextRunTime
        $taskobj.vm = Get-VM -Id $_.Info.TaskObject
        $taskobj.host = $taskobj.vm.Host
        $taskobj.cluster = $taskobj.vm.Host.Parent
        $taskobj.analysis()
        if ($taskobj.issue -eq $TRUE)
        {
          $globalBadTaskList += $taskobj
          $taskobj.dump()
        }
      }
    }
  }
}

# HTML analysis report is generated.
$totalIssues = createAnalysisReport

# csv files for VMs and Hosts are generated.
$VMDetails = VMDetails($globalVMList)
$hostDetails = hostDetails($globalHostList)
$VMDetails | select * | Export-Csv $destDir\vm_summary.csv -NoType
$hostDetails | select * | Export-Csv $destDir\host_summary.csv -NoType

# Get the support bundles for the hosts that we have.
if ($collectLogs)
{
  "Dumping VM support logs for hosts..."
  foreach($VMhost in $globalHostList)
  {
    Get-VMHost -Server $viServer -Name $VMhost.name | Get-Log -Bundle -DestinationPath $destDir
  }
}

if ($vnow)
{
  if ($totalIssues -eq 0)
  {
    "System is correctly configured for vNOW"
  }
  else
  {
    "System failed vNOW checking.  See $destDir\analysis_report.html for details."
  }
}

$viServer | Disconnect-VIServer -confirm:$FALSE


# SIG # Begin signature block
# MIINHAYJKoZIhvcNAQcCoIINDTCCDQkCAQExCzAJBgUrDgMCGgUAMGkGCisGAQQB
# gjcCAQSgWzBZMDQGCisGAQQBgjcCAR4wJgIDAQAABBAfzDtgWUsITrck0sYpfvNR
# AgEAAgEAAgEAAgEAAgEAMCEwCQYFKw4DAhoFAAQU9hufAO3/JXov4gpAIdYW8xAA
# RVegggpeMIIFJjCCBA6gAwIBAgIQDLQuovfB8WUTTd+P6fRDhzANBgkqhkiG9w0B
# AQsFADByMQswCQYDVQQGEwJVUzEVMBMGA1UEChMMRGlnaUNlcnQgSW5jMRkwFwYD
# VQQLExB3d3cuZGlnaWNlcnQuY29tMTEwLwYDVQQDEyhEaWdpQ2VydCBTSEEyIEFz
# c3VyZWQgSUQgQ29kZSBTaWduaW5nIENBMB4XDTE3MTExNTAwMDAwMFoXDTIwMTEx
# OTEyMDAwMFowYzELMAkGA1UEBhMCR0IxEDAOBgNVBAcTB0VuZmllbGQxIDAeBgNV
# BAoTF01ldGFzd2l0Y2ggTmV0d29ya3MgTHRkMSAwHgYDVQQDExdNZXRhc3dpdGNo
# IE5ldHdvcmtzIEx0ZDCCASIwDQYJKoZIhvcNAQEBBQADggEPADCCAQoCggEBAL9o
# iZ/QgqEoj0DK8G+pBIXLpwM2JDgpBDKiPd20Ufht/aCCXdxNkwhgGmT4OGZX+04i
# O+lqE2ueNnxySpxDV0Cj7dfx71umx2vQhQ+hD50ZG/1996hQES4BGGmeNC/gAL33
# xQCO+AUCLeIjfvGdQ9dZa2o4Ok4VBF5aSJoy5V6sSSPCb388nrvvktzHgT82gufp
# 7t5MfQq3NsKpovNGjcH3ens5XuZDb+43GSytKPSuX9Mjsy5Bkj83PXUy9OO1mgcZ
# 8t2R4cQHb1eCUaEqJvGuPLIos9GvbWwskaW8imlL8w1A+WAr3DQXdtnTFzipjbMP
# PMmBHp6ZgNI5jjzMINMCAwEAAaOCAcUwggHBMB8GA1UdIwQYMBaAFFrEuXsqCqOl
# 6nEDwGD5LfZldQ5YMB0GA1UdDgQWBBTr9IEPehl9ZiocM+siSeJTe8P7BTAOBgNV
# HQ8BAf8EBAMCB4AwEwYDVR0lBAwwCgYIKwYBBQUHAwMwdwYDVR0fBHAwbjA1oDOg
# MYYvaHR0cDovL2NybDMuZGlnaWNlcnQuY29tL3NoYTItYXNzdXJlZC1jcy1nMS5j
# cmwwNaAzoDGGL2h0dHA6Ly9jcmw0LmRpZ2ljZXJ0LmNvbS9zaGEyLWFzc3VyZWQt
# Y3MtZzEuY3JsMEwGA1UdIARFMEMwNwYJYIZIAYb9bAMBMCowKAYIKwYBBQUHAgEW
# HGh0dHBzOi8vd3d3LmRpZ2ljZXJ0LmNvbS9DUFMwCAYGZ4EMAQQBMIGEBggrBgEF
# BQcBAQR4MHYwJAYIKwYBBQUHMAGGGGh0dHA6Ly9vY3NwLmRpZ2ljZXJ0LmNvbTBO
# BggrBgEFBQcwAoZCaHR0cDovL2NhY2VydHMuZGlnaWNlcnQuY29tL0RpZ2lDZXJ0
# U0hBMkFzc3VyZWRJRENvZGVTaWduaW5nQ0EuY3J0MAwGA1UdEwEB/wQCMAAwDQYJ
# KoZIhvcNAQELBQADggEBANN8jMgUji4sIQKDq+pVLzE8QNCxybLx0JPMoB1MWwKj
# 43fbASkuf2TR6gzIfsmrOmYmEH8bU9zHXQ09xXCjs5eoXW9lzkr16WP0vl77hd8j
# tQpKTwOEoBIFEFVI3ampGf2K2T0/7QqZfcnt1NzAJGX+VEB+KV5XO/mAl1zVB/tD
# hGdztc35GtqCgTuPDBvNu530JbYPiHNPiXMnGnG7KUeaFBqHhHt0gL6ls5Hpj/ro
# NDhCGmJw1VxOEikpCkpIUyZ1DdJ9ZBgqRFSObfTk7QTLJJMkjuQTd5R6gEJD4c1z
# WNUc0P223tTkZR4F3DbV6EBy6F2SzNN05AbNQ40UdbAwggUwMIIEGKADAgECAhAE
# CRgbX9W7ZnVTQ7VvlVAIMA0GCSqGSIb3DQEBCwUAMGUxCzAJBgNVBAYTAlVTMRUw
# EwYDVQQKEwxEaWdpQ2VydCBJbmMxGTAXBgNVBAsTEHd3dy5kaWdpY2VydC5jb20x
# JDAiBgNVBAMTG0RpZ2lDZXJ0IEFzc3VyZWQgSUQgUm9vdCBDQTAeFw0xMzEwMjIx
# MjAwMDBaFw0yODEwMjIxMjAwMDBaMHIxCzAJBgNVBAYTAlVTMRUwEwYDVQQKEwxE
# aWdpQ2VydCBJbmMxGTAXBgNVBAsTEHd3dy5kaWdpY2VydC5jb20xMTAvBgNVBAMT
# KERpZ2lDZXJ0IFNIQTIgQXNzdXJlZCBJRCBDb2RlIFNpZ25pbmcgQ0EwggEiMA0G
# CSqGSIb3DQEBAQUAA4IBDwAwggEKAoIBAQD407Mcfw4Rr2d3B9MLMUkZz9D7RZmx
# OttE9X/lqJ3bMtdx6nadBS63j/qSQ8Cl+YnUNxnXtqrwnIal2CWsDnkoOn7p0WfT
# xvspJ8fTeyOU5JEjlpB3gvmhhCNmElQzUHSxKCa7JGnCwlLyFGeKiUXULaGj6Ygs
# IJWuHEqHCN8M9eJNYBi+qsSyrnAxZjNxPqxwoqvOf+l8y5Kh5TsxHM/q8grkV7tK
# tel05iv+bMt+dDk2DZDv5LVOpKnqagqrhPOsZ061xPeM0SAlI+sIZD5SlsHyDxL0
# xY4PwaLoLFH3c7y9hbFig3NBggfkOItqcyDQD2RzPJ6fpjOp/RnfJZPRAgMBAAGj
# ggHNMIIByTASBgNVHRMBAf8ECDAGAQH/AgEAMA4GA1UdDwEB/wQEAwIBhjATBgNV
# HSUEDDAKBggrBgEFBQcDAzB5BggrBgEFBQcBAQRtMGswJAYIKwYBBQUHMAGGGGh0
# dHA6Ly9vY3NwLmRpZ2ljZXJ0LmNvbTBDBggrBgEFBQcwAoY3aHR0cDovL2NhY2Vy
# dHMuZGlnaWNlcnQuY29tL0RpZ2lDZXJ0QXNzdXJlZElEUm9vdENBLmNydDCBgQYD
# VR0fBHoweDA6oDigNoY0aHR0cDovL2NybDQuZGlnaWNlcnQuY29tL0RpZ2lDZXJ0
# QXNzdXJlZElEUm9vdENBLmNybDA6oDigNoY0aHR0cDovL2NybDMuZGlnaWNlcnQu
# Y29tL0RpZ2lDZXJ0QXNzdXJlZElEUm9vdENBLmNybDBPBgNVHSAESDBGMDgGCmCG
# SAGG/WwAAgQwKjAoBggrBgEFBQcCARYcaHR0cHM6Ly93d3cuZGlnaWNlcnQuY29t
# L0NQUzAKBghghkgBhv1sAzAdBgNVHQ4EFgQUWsS5eyoKo6XqcQPAYPkt9mV1Dlgw
# HwYDVR0jBBgwFoAUReuir/SSy4IxLVGLp6chnfNtyA8wDQYJKoZIhvcNAQELBQAD
# ggEBAD7sDVoks/Mi0RXILHwlKXaoHV0cLToaxO8wYdd+C2D9wz0PxK+L/e8q3yBV
# N7Dh9tGSdQ9RtG6ljlriXiSBThCk7j9xjmMOE0ut119EefM2FAaK95xGTlz/kLEb
# Bw6RFfu6r7VRwo0kriTGxycqoSkoGjpxKAI8LpGjwCUR4pwUR6F6aGivm6dcIFzZ
# cbEMj7uo+MUSaJ/PQMtARKUT8OZkDCUIQjKyNookAv4vcn4c10lFluhZHen6dGRr
# sutmQ9qzsIzV6Q3d9gEgzpkxYz0IGhizgZtPxpMQBvwHgfqL2vmCSfdibqFT+hKU
# GIUukpHqaGxEMrJmoecYpJpkUe8xggIoMIICJAIBATCBhjByMQswCQYDVQQGEwJV
# UzEVMBMGA1UEChMMRGlnaUNlcnQgSW5jMRkwFwYDVQQLExB3d3cuZGlnaWNlcnQu
# Y29tMTEwLwYDVQQDEyhEaWdpQ2VydCBTSEEyIEFzc3VyZWQgSUQgQ29kZSBTaWdu
# aW5nIENBAhAMtC6i98HxZRNN34/p9EOHMAkGBSsOAwIaBQCgeDAYBgorBgEEAYI3
# AgEMMQowCKACgAChAoAAMBkGCSqGSIb3DQEJAzEMBgorBgEEAYI3AgEEMBwGCisG
# AQQBgjcCAQsxDjAMBgorBgEEAYI3AgEVMCMGCSqGSIb3DQEJBDEWBBRrgsfGJNr6
# hrEmo9Gh0shX13AScTANBgkqhkiG9w0BAQEFAASCAQAavR190BXv7nL9LttIworD
# iJYV5HqH5F9qL+YjEg/7R6ccRfCO6CoFmqSiY8U8u0vhKMAsC379Oc7gyp+TdFyn
# piNTb+ESbz0RNCZY4a6DhgDoBzqc0MoZhJ0jCgK6Qj47g/g6cV+aDFgU9leTw0y2
# WU3Bl+rXKWdxAv8CaTXTKWyaNp8QwUw00BX+AuOyCWLyUMLMqzxqbmLUeWCo/imK
# ICu8x0gWzvrLRzfKk6v+tUo43ylC91lSDySSU5tFGQXr5x8IZLzE9t1CoQftMzrQ
# zG6CULHpar8sHZHIplnLIwp7TvDBF0G/IeRSuFf305dpqkXfdmAFY5cSVw8FKIEw
# SIG # End signature block
